/***************************************************************************
                           feedbacklistbox.cpp
                           version 0.5.2 $Revision: 1.6 $
                           -------------------------------
    copyright            : (C) 2006-2007 by Guido Scholz
    email                : guido.scholz@bayernline.de
    last modified        : $Date: 2009-03-16 17:38:04 $
****************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
   This code handles a list of feedback modules
 ***************************************************************************/

#include <QApplication>
#include <QDrag>
#include <QMimeData>
#include <QPoint>
#include <QMouseEvent>

#include "feedbacklistbox.h"

#include "pixmaps/feedback_red.xpm"
#include "pixmaps/feedback_white.xpm"


FeedbackListBox::FeedbackListBox(QWidget* parent,
        const char* name, unsigned int b): QListWidget(parent),
    bus(b), mousePressed(false)
{
    setObjectName(name);
    setViewMode(QListView::IconMode);
    setIconSize(QSize(140, 80));
    setSelectionMode(NoSelection);
    setUniformItemSizes(true);
    moduleType = FeedbackModule::fbm16;
}

void FeedbackListBox::updateModuleSetup(int mt, unsigned int mc)
{
    if (mt != moduleType) {
        moduleType = (FeedbackModule::ModuleType) mt;
        updateModulesType();
        //triggerUpdate(true);
    }

    if (count() != (int)mc)
        updateModuleNumber(mc);
}

void FeedbackListBox::updateModulesType()
{
    FeedbackModule* fbm;

    if (count() > 0) {
        for (int i = 0; i < count(); ++i) {
            fbm = (FeedbackModule*) item(i);
            if (fbm != NULL) {
                fbm->setModuleType(moduleType);
                /*TODO: repaint item
                  updateItem(fbm);*/
            }
        }
    }
}

void FeedbackListBox::updateModuleNumber(unsigned int mc)
{
    int delta = 0;
    FeedbackModule* fbm;

    if (count() > (int) mc) {
        // delete some modules
        delta = count() - mc;
        for (int i = 0; i < delta; i++) {
            fbm = (FeedbackModule*) takeItem(count() - 1);
            if (fbm != NULL)
                delete fbm;
        }
    }
    else if (count() < (int) mc) {
        // add some modules
        delta = mc - count();
        for (int i = 0; i < delta; i++) {
            fbm = new FeedbackModule(this);
            fbm->setId(count());
            fbm->setModuleType(moduleType);
        }
    }
}

void FeedbackListBox::updateFeedbackPortState(unsigned int contact, bool state)
{
    FeedbackModule* fbm;

    if (count() > 0) {
        for (int i = 0; i < count(); ++i) {
            fbm = (FeedbackModule*) item(i);
            if (fbm != NULL) {
                if (fbm->changeFeedbackState(contact, state)) {
                    /*TODO: repaint item
                      updateItem(fbm);*/
                    break;
                }
            }
        }
    }
}


void FeedbackListBox::mousePressEvent(QMouseEvent* e)
{
    QListWidget::mousePressEvent(e);
    QPoint p = e->pos();
    QListWidgetItem* i = itemAt(p);
    if (i == NULL)
        return;

    QRect r = visualItemRect(i);
    if (dynamic_cast<FeedbackModule*>(i)->isContactPosition(p - r.topLeft())) {
        presspos = p;
        mousePressed = true;
    }
}


void FeedbackListBox::mouseReleaseEvent(QMouseEvent*)
{
    mousePressed = false;
}


void FeedbackListBox::mouseMoveEvent(QMouseEvent* e)
{
    if (!mousePressed)
        return;

    if ((presspos - e->pos()).manhattanLength() >
            QApplication::startDragDistance()) {

        mousePressed = false;
        QListWidgetItem* i = itemAt(presspos);
        if (i == NULL)
            return;

        QByteArray data;
        bool occupied = dynamic_cast<FeedbackModule*>(i)->getContactData(data);
        if (data.size() > 0) {
            QMimeData *mimeData = new QMimeData;
            mimeData->setData(MIME_FBC, data);

            QDrag* d = new QDrag(this);
            d->setMimeData(mimeData);

            if (occupied)
                d->setPixmap(QPixmap(feedback_red_xpm));
            else
                d->setPixmap(QPixmap(feedback_white_xpm));
            d->exec(Qt::MoveAction);
        }
    }
}


