/***************************************************************************
                           keyboard.cpp
                           version 0.5.6 $Revision: 1.24 $
                           -------------------------------
    copyright            : (C) 1999-2003 by Stefan Preis
                         : (C) 2004-2009 Guido Scholz
    email                : guido.scholz@bayernline.de
    last modified        : $Date: 2009-10-27 20:54:53 $
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
   this code shows a window with a manual keyboard to switch solenoids
 ***************************************************************************/

#include <QApplication>
#include <QLabel>
#include <QLayout>
#include <QPushButton>
#include <QValidator>

#include "resources.h"
#include "keyboard.h"
#include "preferences.h"



keyboard::keyboard(int protocol,
        QWidget* parent, const char* name): QDialog(parent)
{
    setObjectName("name");
    setWindowTitle(tr("Keyboard"));

    QBoxLayout* baseLayout = new QVBoxLayout(this);
    baseLayout->setSpacing(10);

    /*line with decoder combobox*/
    QHBoxLayout* protocolLayout = new QHBoxLayout(this);
    protocolLayout->setSpacing(6);
    baseLayout->addLayout(protocolLayout);

    QLabel* protocolLbl = new QLabel(tr("&Protocol:"), this);
    protocolLayout->addWidget(protocolLbl);
    protocolLayout->addStretch();

    protocolCB = new QComboBox(this);
    protocolLayout->addWidget(protocolCB);
    protocolCB->addItem("MM");
    protocolCB->addItem("DCC");
    protocolCB->addItem("Slx");
    protocolCB->addItem("Srv");
    protocolLbl->setBuddy(protocolCB);
    protocolCB->setCurrentIndex(protocol);
    connect(protocolCB, SIGNAL(activated(int)),
            this, SIGNAL(protocolSelected(int)));
    protocolCB->setToolTip(tr("Select the decoder protocol\n"
                "MM: Maerklin/Motorola\n"
                "DCC: NMRA/DCC\n"
                "Slx: Selectrix\n"
                "Srv: Protocol by server\n"));

    /*line with SRCP-bus label and edit line*/
    QBoxLayout* busLayout = new QHBoxLayout(this);
    busLayout->setSpacing(6);
    baseLayout->addLayout(busLayout);

    QLabel *busLbl = new QLabel(tr("SRCP-&Bus:"), this);
    busLayout->addWidget(busLbl);

    busLayout->addStretch();

    busLE = new QLineEdit("1", this);
    QFontMetrics fm(busLE->font());
    int LEwidth = fm.width("8888") + 10;
    busLE->setMaxLength(4);
    busLE->setMaximumWidth(LEwidth);
    QValidator* busValidator = new QIntValidator(1, 999, this);
    busLE->setValidator(busValidator);
    busLayout->addWidget(busLE);
    busLbl->setBuddy(busLE);
    busLE->setToolTip(tr("Enter the SRCP bus for the address"));

    /*line with address label and edit line*/
    QBoxLayout* addressLayout = new QHBoxLayout(this);
    addressLayout->setSpacing(6);
    baseLayout->addLayout(addressLayout);

    QLabel *labelAddress = new QLabel(tr("&Address:"), this);
    addressLayout->addWidget(labelAddress);

    addressLayout->addStretch();

    addressLE = new QLineEdit("1", this);
    addressLE->setMaxLength(4);
    addressLE->setMaximumWidth(LEwidth);
    QValidator* addressValidator = new QIntValidator(1, MAX_GADCC, this);
    addressLE->setValidator(addressValidator);
    addressLayout->addWidget(addressLE);
    labelAddress->setBuddy(addressLE);
    addressLE->setToolTip(tr("Enter the address to be switched"));

    /*line with red and green buttons*/
    QBoxLayout* buttonLayout = new QHBoxLayout(this);
    buttonLayout->setSpacing(6);
    baseLayout->addLayout(buttonLayout);

    buttonLayout->addStretch();

    QPushButton* redPB = new QPushButton("&0", this);
    redPB->setMaximumWidth(LEwidth);
    //TODO: check setAutoFillBackground ( bool enabled )
    redPB->setPalette(QPalette(QColor(255, 0, 0)));
    connect(redPB, SIGNAL(clicked()), this, SLOT(slotActivateRed()));
    buttonLayout->addWidget(redPB);
    redPB->setToolTip(tr("Press this button to activate red connector"));

    buttonLayout->addStretch();

    QPushButton* greenPB = new QPushButton("&1", this);
    greenPB->setMaximumWidth(LEwidth);
    greenPB->setPalette(QPalette(QColor(0, 255, 0)));
    connect(greenPB, SIGNAL(clicked()), this, SLOT(slotActivateGreen()));
    buttonLayout->addWidget(greenPB);
    greenPB->setDefault(true);
    greenPB->setToolTip(
            tr("Press this button to activate green connector"));

    buttonLayout->addStretch();
}


// send command for selected protocol
void keyboard::slotActivateRed()
{
    GaMessage::Protocol protocol;
    unsigned int adr = addressLE->text().toUInt();
    unsigned int bus = busLE->text().toUInt();

    switch(protocolCB->currentIndex()) {
        case 0:
            protocol = GaMessage::proMM;
            break;
        case 1:
            protocol = GaMessage::proDCC;
            break;
        case 2:
            protocol = GaMessage::proSelectrix;
            break;
        case 3:
            protocol = GaMessage::proServer;
            break;
        default:
            protocol = GaMessage::proMM;
    }

    GaMessage sm = GaMessage(SrcpMessage::mtGaSet, bus, adr, protocol,
            0, 1, 200);

    // send also init message
    sm.setType(SrcpMessage::mtGaInit);
    emit sendSrcpMessage(&sm);
    // give time to show effect
    qApp->processEvents();
    sm.setType(SrcpMessage::mtGaSet);

    emit sendSrcpMessage(&sm);
}


void keyboard::slotActivateGreen()
{
    GaMessage::Protocol protocol;
    unsigned int adr = addressLE->text().toUInt();
    unsigned int bus = busLE->text().toUInt();

    switch(protocolCB->currentIndex()) {
        case 0:
            protocol = GaMessage::proMM;
            break;
        case 1:
            protocol = GaMessage::proDCC;
            break;
        case 2:
            protocol = GaMessage::proSelectrix;
            break;
        case 3:
            protocol = GaMessage::proServer;
            break;
        default:
            protocol = GaMessage::proMM;
    }

    GaMessage sm = GaMessage(SrcpMessage::mtGaSet, bus, adr, protocol,
            1, 1, 200);

    // send also init message
    sm.setType(SrcpMessage::mtGaInit);
    emit sendSrcpMessage(&sm);
    // give time to show effect
    qApp->processEvents();
    sm.setType(SrcpMessage::mtGaSet);

    emit sendSrcpMessage(&sm);
}

