/***************************************************************************
                           mainwindow.cpp
                           -------------------------------
    Copyright            : (C) 1999-2003 by Stefan Preis
                         : (C) 2004-2009 Guido Scholz
    email                : guido.scholz@bayernline.de
    last modified        : $Date: 2009-12-20 20:05:49 $
                           $Revision: 1.174 $
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
   this file provides a menubar, a toolbar, and a scrollable view for the
   elements and sets up the connection to the SRCP server
 ***************************************************************************/

#include <qmenubar.h>
#include <qprocess.h>
#include <qtextcodec.h>
#include <QLocale>
#include <QToolBar>
#include <QSettings>

#include "aboutdialog.h"
#include "commandport.h"
#include "crcfmessage.h"
#include "element.h"
#include "feedbackviewer.h"
#include "finder.h"
#include "gbsarea.h"
#include "infoport.h"
#include "keyboard.h"
#include "layouteditmodeagrp.h"
#include "mainwindow.h"
#include "messagehistory.h"
#include "newlayoutdialog.h"
#include "options.h"
#include "paintitemwindow.h"
#include "preferences.h"
#include "resources.h"
#include "routelistwindow.h"
#include "serverinfodialog.h"
#include "trainnumberdialog.h"


#include "../icons/spdrs60_32.xpm"
/*toolbar icons*/
#include "pixmaps/filenew.xpm"
#include "pixmaps/fileopen.xpm"
#include "pixmaps/filesave.xpm"
#include "pixmaps/filesaveas.xpm"
#include "pixmaps/filenewwindow.xpm"
#include "pixmaps/fileclose.xpm"
#include "pixmaps/filequit.xpm"

#include "pixmaps/editcut.xpm"
#include "pixmaps/editcopy.xpm"
#include "pixmaps/editpaste.xpm"
#include "pixmaps/editfind.xpm"
#include "pixmaps/editoptions.xpm"

#include "pixmaps/daemonconnect.xpm"
#include "pixmaps/daemondisconnect.xpm"
#include "pixmaps/daemonkill.xpm"
#include "pixmaps/daemonreset.xpm"
#include "pixmaps/daemoninfo.xpm"

#include "pixmaps/viewroute.xpm"
#include "pixmaps/viewfeedback.xpm"
#include "pixmaps/viewtrainnumber.xpm"
#include "pixmaps/viewkeyboard.xpm"
#include "pixmaps/viewclock.xpm"
#include "pixmaps/viewnormalmode.xpm"
#include "pixmaps/viewlayouteditmode.xpm"
#include "pixmaps/viewrouteeditmode.xpm"

#include "pixmaps/layoutstart.xpm"
#include "pixmaps/layoutstop.xpm"
#include "pixmaps/layoutnotrot.xpm"

#include "pixmaps/route_start.xpm"
#include "pixmaps/route_stop.xpm"
#include "pixmaps/route_edit.xpm"
#include "pixmaps/route_new.xpm"
#include "pixmaps/route_copy.xpm"
#include "pixmaps/route_clear.xpm"

/*some constants for data files*/
static const char GF_CMDHOST[]       = "cmdhost";
static const char GF_FORMATVERSION[] = "formatversion";
static const char GF_FV[]            = "3";
static const char GF_RWCC[]          = "rwcc";

/*string constants for personal config file*/
static const char KS[]              = "=";
static const char CF_DS[]           = ":";
static const char CF_SHOWHP2[]      = "showhp2";
static const char CF_TURNOUTBLINK[] = "blinkingturnouts";
static const char CF_TOOLTIPS[]     = "tooltips";
static const char CF_DATATOOLTIPS[] = "datatooltips";
static const char CF_ADDRESSLABEL[] = "addresslabel";
static const char CF_INITSIGNALS[]  = "initsignals";
static const char CF_LAYOUTCOLS[]   = "layoutcolumns";
static const char CF_LAYOUTROWS[]   = "layoutrows";
static const char CF_SENDSTATE[]    = "sendstate";
static const char CF_CONVERTTIME[]  = "converttime";
static const char CF_SHOWTIME[]     = "showtime";
static const char CF_FULLSCREEN[]   = "fullscreen";
static const char CF_AUTOLOAD[]     = "autoload";
static const char CF_AUTOLAYOUT[]   = "autolayout";
static const char CF_AUTOSAVE[]     = "autosave";
static const char CF_EDITOR[]       = "editor";
static const char CF_BROWSER[]      = "browser";
static const char CF_PROTOCOL[]     = "protocol";
static const char CF_ACTIVETIME[]   = "activationtime";
static const char CF_AUTOTTDIR[]    = "autoturntabledir";
static const char CF_TTROUNDTIME[]  = "turntableroundtime";
static const char CF_ROUTINGTIME[]  = "routingtime";
static const char CF_FEEDBACKTYPE[] = "feedbacktype";
static const char CF_FBMODSIZE[]    = "feedbackmodulesize";
static const char CF_FBMODTYPE[]    = "feedbackmoduletype";
static const char CF_FBBUS1[]       = "fbbus1";
static const char CF_FBBUS2[]       = "fbbus2";
static const char CF_FBBUS3[]       = "fbbus3";
static const char CF_FBBUS4[]       = "fbbus4";
static const char CF_LASTDIR[]      = "lastdir";
static const char CF_KEYBOARD[]     = "keyboardprotocol";
static const char CF_GMROUTESTATE[] = "gmroutestate";
static const char CF_GMTRAINNUMBER[] = "gmtrainnumber";
static const char CF_RECENTFILE[]   = "recentfile";

// user preferences/program init filename
static const char SPDRS60_INIT[] = ".spdrs60rc";

// make sure only the first window gets group leader
static bool firstinstance = true;


MainWindow::MainWindow(): QMainWindow(NULL),
                          visualMode(kvmNormal),
                          isFBInitMode(true),
                          rwccid(0),
                          rwccname(tr("Untitled")),
                          rtViewer(NULL),
                          router(NULL),
                          keybWindow(NULL),
                          trainnumberdialog(NULL),
                          /*Networking */
                          SRCPCommandState(srcpUndefined),
                          cmdAutoLogin(false),
                          cmdAutoPower(false),
                          cmdAutoSendAll(false)
{
    QWidget::setAttribute(Qt::WA_DeleteOnClose);
    setWindowIcon(QPixmap(spdrs60_32));

    lastDir = QDir::homePath();  // remembers path for FileOpen

    // clear global filedialog settings to avoid cross application
    // spying
    QSettings settings( QSettings::UserScope, "Trolltech");
    settings.beginGroup("Qt");
    settings.remove("filedialog");

    // init two sockets with default values, feedback socket for SRCP
    // 0.7 communication is created on demand
    commandPort = new CommandPort(this, "commandPort");
    Q_CHECK_PTR(commandPort);

    connect(commandPort, SIGNAL(connectionStateChanged(bool)),
            this, SLOT(updateCommandConnectionState(bool)));
    connect(commandPort, SIGNAL(messageReceived(const SrcpMessage*)),
            this, SLOT(processCommandMessage(const SrcpMessage*)));


    infoPort = new InfoPort(this, "infoPort");
    Q_CHECK_PTR(infoPort);

    connect(infoPort, SIGNAL(connectionStateChanged(bool)),
            this, SLOT(updateInfoConnectionState(bool)));
    connect(infoPort, SIGNAL(messageReceived(const SrcpMessage*)),
            this, SLOT(processInfoMessage(const SrcpMessage*)));

    // setup main window with all menus
    initMainWindow();
    // read user dependend config file
    readConfigFile();

    if (pref.fullscreen)
        slotViewToggleFullscreen();

    statusMessage(tr("Program successfully started!"));
}

/* Cleanup by destructor */
MainWindow::~MainWindow()
{
    // send LOGOUT command to daemon
    closeSrcpServerConnection();
    writeConfigFile();
}

/**
 * read application settings from config file, this is typicaly
 * done on application startup
 */
void MainWindow::readConfigFile()
{
    QFile file(QDir::homePath() + "/" + SPDRS60_INIT);
    if (!file.open(QIODevice::ReadOnly)) {
        /* if no configuration file is found, just keep defaults */
        statusMessage(tr("User preferences file not found") + ": ~/" +
                   SPDRS60_INIT);
        return;
    }
    QTextStream ts(&file);
    QString s, key, value;

    while (!ts.atEnd()) {
        s = ts.readLine();
        if (!s.startsWith("#")) {
            key = s.section(KS, 0, 0);
            value = s.section(KS, 1).trimmed();

            if (key.compare(CF_SHOWHP2) == 0) {
                pref.hp2 = value.toInt();
            }
            else if (key.compare(CF_TURNOUTBLINK) == 0) {
                pref.blinkingturnouts = value.toInt();
            }
            else if (key.compare(CF_DATATOOLTIPS) == 0) {
                pref.datatooltips = value.toInt();
            }
            else if (key.compare(CF_ADDRESSLABEL) == 0) {
                pref.addresslabeling = value.toInt();
            }
            else if (key.compare(CF_INITSIGNALS) == 0) {
                pref.initsignalsred = value.toInt();
            }
            else if (key.compare(CF_LAYOUTCOLS) == 0) {
                pref.layoutcols = value.toUInt();
            }
            else if (key.compare(CF_LAYOUTROWS) == 0) {
                pref.layoutrows = value.toUInt();
            }
            else if (key.compare(CF_SENDSTATE) == 0) {
                pref.sendstate = value.toInt();
            }
            else if (key.compare(CF_CONVERTTIME) == 0) {
                pref.converttime = value.toInt();
            }
            else if (key.compare(CF_SHOWTIME) == 0) {
                pref.showtime = value.toInt();
            }
            else if (key.compare(CF_FULLSCREEN) == 0) {
                pref.fullscreen = value.toInt();
            }
            else if (key.compare(CF_AUTOLOAD) == 0) {
                pref.autoload = value.toInt();
            }
            else if (key.compare(CF_AUTOLAYOUT) == 0) {
                pref.autolayout = value;
            }
            else if (key.compare(CF_AUTOSAVE) == 0) {
                pref.autosave = value.toInt();
            }
            else if (key.compare(CF_EDITOR) == 0) {
                pref.editor = value;
            }
            else if (key.compare(CF_BROWSER) == 0) {
                pref.browser = value;
            }
            else if (key.compare(CF_PROTOCOL) == 0) {
                pref.protocol = 2;
                if (value == "DCC")
                    pref.protocol = 0;
                else if (value == "Motorola")
                    pref.protocol = 1;
                else if (value == "Selectrix")
                    pref.protocol = 3;
            }
            else if (key.compare(CF_ACTIVETIME) == 0) {
                pref.activetime = value.toInt();
            }
            else if (key.compare(CF_AUTOTTDIR) == 0) {
                pref.autottdir = value.toInt();
            }
            else if (key.compare(CF_TTROUNDTIME) == 0) {
                pref.ttroundtime = value.toDouble();
            }
            else if (key.compare(CF_ROUTINGTIME) == 0) {
                pref.routingtime = value.toInt();
            }
            //TODO: remove before release
            else if (key.compare(CF_FEEDBACKTYPE) == 0) {
                pref.fbfactor = value.toInt();
            }
            else if (key.compare(CF_FBMODSIZE) == 0) {
                pref.fbfactor = value.toInt();
            }
            else if (key.compare(CF_FBMODTYPE) == 0) {
                pref.fbmoduletype = value.toInt();
            }
            else if (key.compare(CF_FBBUS1) == 0) {
                pref.fbbus1.number = value.section(CF_DS, 0, 0).toUInt();
                pref.fbbus1.modules = value.section(CF_DS, 1, 1).toUInt();
            }
            else if (key.compare(CF_FBBUS2) == 0) {
                pref.fbbus2.number = value.section(CF_DS, 0, 0).toUInt();
                pref.fbbus2.modules = value.section(CF_DS, 1, 1).toUInt();
            }
            else if (key.compare(CF_FBBUS3) == 0) {
                pref.fbbus3.number = value.section(CF_DS, 0, 0).toUInt();
                pref.fbbus3.modules = value.section(CF_DS, 1, 1).toUInt();
            }
            else if (key.compare(CF_FBBUS4) == 0) {
                pref.fbbus4.number = value.section(CF_DS, 0, 0).toUInt();
                pref.fbbus4.modules = value.section(CF_DS, 1, 1).toUInt();
            }
            else if (key.compare(CF_LASTDIR) == 0) {
                lastDir = value.trimmed();
                // check if directory is valid
                if (!QFile::exists(lastDir))
                    lastDir = QDir::homePath();
            }
            else if (key.compare(CF_KEYBOARD) == 0) {
                pref.keyboardprot = value.toInt();
            }
            else if (key.compare(CF_GMROUTESTATE) == 0) {
                pref.gmbroadcastroutestate = value.toInt();
            }
            else if (key.compare(CF_GMTRAINNUMBER) == 0) {
                pref.gmbroadcasttrainnumber = value.toInt();
            }
            else if (key.compare(CF_RECENTFILE) == 0) {
                addRecentlyOpenedFile(value, recentFiles);
            }
        }
    }
    file.close();
    //tell the feedback viewer about changed feedback module layout
    fbViewer->updateBusAndModuleStructure();
}


/**
 * write application settings to user config file, this is typicaly
 * done when application window is closed
 */
void MainWindow::writeConfigFile()
{
    QFile file(QDir::homePath() + "/" + SPDRS60_INIT);

    if (!file.open(QIODevice::WriteOnly)) {
        statusMessage(tr("Error: Could not save configuration"
                    " file: ~/%1").arg(SPDRS60_INIT));
        return;
    }
    statusMessage(tr("Writing %1 configuration file: ~/%2")
            .arg(PACKAGE).arg(SPDRS60_INIT));

    QDateTime dt = QDateTime::currentDateTime();
    QTextStream ts(&file);

    QString rtstr;
    rtstr.sprintf("%.2f", pref.ttroundtime);

    ts  << "# " PACKAGE " configuration file" << endl
        << "# last modified: " << dt.toString(Qt::ISODate) << endl
        << "#" << endl
        << CF_SHOWHP2      << KS << (int) pref.hp2 << endl
        << CF_TURNOUTBLINK << KS << (int) pref.blinkingturnouts << endl
        << CF_DATATOOLTIPS << KS << (int) pref.datatooltips << endl
        << CF_ADDRESSLABEL << KS << (int) pref.addresslabeling << endl
        << CF_INITSIGNALS  << KS << (int) pref.initsignalsred << endl
        << CF_LAYOUTCOLS   << KS << pref.layoutcols << endl
        << CF_LAYOUTROWS   << KS << pref.layoutrows << endl
        << CF_SENDSTATE    << KS << pref.sendstate << endl
        << CF_CONVERTTIME  << KS << pref.converttime << endl
        << CF_FULLSCREEN   << KS << (int) pref.fullscreen << endl
        << CF_AUTOLOAD     << KS << (int) pref.autoload << endl
        << CF_AUTOLAYOUT   << KS << pref.autolayout << endl
        << CF_AUTOSAVE     << KS << (int) pref.autosave << endl
        << CF_EDITOR       << KS << pref.editor << endl
        << CF_BROWSER      << KS << pref.browser << endl
        << CF_PROTOCOL     << KS <<
            ((pref.protocol == 1) ? "Motorola"
            : (pref.protocol == 0) ? "DCC"
            : (pref.protocol == 3) ? "Selectrix" : "Server") << endl
        << CF_ACTIVETIME   << KS << pref.activetime << endl
        << CF_AUTOTTDIR    << KS << (int) pref.autottdir << endl
        << CF_TTROUNDTIME  << KS << rtstr << endl
        << CF_ROUTINGTIME  << KS << pref.routingtime << endl
        << CF_FBMODSIZE    << KS << pref.fbfactor << endl
        << CF_FBMODTYPE    << KS << pref.fbmoduletype << endl
        << CF_FBBUS1       << KS << pref.fbbus1.number
                           << CF_DS << pref.fbbus1.modules << endl
        << CF_FBBUS2       << KS << pref.fbbus2.number
                           << CF_DS << pref.fbbus2.modules << endl
        << CF_FBBUS3       << KS << pref.fbbus3.number
                           << CF_DS << pref.fbbus3.modules << endl
        << CF_FBBUS4       << KS << pref.fbbus4.number
                           << CF_DS << pref.fbbus4.modules << endl
        << CF_LASTDIR      << KS << lastDir << endl
        << CF_KEYBOARD     << KS << pref.keyboardprot << endl
        << CF_GMROUTESTATE << KS << pref.gmbroadcastroutestate << endl
        << CF_GMTRAINNUMBER << KS << pref.gmbroadcasttrainnumber << endl;

    for (int i = recentFiles.size() - 1; i >= 0; --i)
        ts << CF_RECENTFILE << KS << recentFiles.at(i) << endl;

    file.close();
}

/*
 * This is the window layout in detail:
 +---------------------------------------------------------+
 |       QMainWindow                                       |
 | +-----------------------------------------------------+ |
 | |     QVBox                                           | |
 | | +-------------------------------------------------+ | |
 | | |   QScrollView                                   | | |
 | | | +---------------------------------------------+ | | |
 | | | | GBSArea                                     | | | |
 | | | |                                             | | | |
 | | | |                                             | | | |
 | | | |                                             | | | |
 | | | |                                             | | | |
 | | | |                                             | | | |
 | | | |                                             | | | |
 | | | |                                             | | | |
 | | | +---------------------------------------------+ | | |
 | | +-------------------------------------------------+ | |
 | +-----------------------------------------------------+ |
 | | +-------------------------------------------------+ | |
 | | |  MessageHistory                                 | | |
 | | +-------------------------------------------------+ | |
 | +-----------------------------------------------------+ |
 +---------------------------------------------------------+
 */
void MainWindow::initMainWindow()
{
    QVBoxLayout *layout = new QVBoxLayout;
    layout->setSpacing(0);
    layout->setMargin(0);

    QWidget *vBox = new QWidget();
    vBox->setLayout(layout);

    /*gbs*/
    gbs = new GBSArea();
    Q_CHECK_PTR(gbs);

    QScrollArea* scrollview = new QScrollArea();
    QPalette p(scrollview->palette());
    p.setColor(scrollview->backgroundRole(), Qt::darkGray);
    scrollview->setPalette(p);
    scrollview->setWidget(gbs);
    layout->addWidget(scrollview);

    connect(this, SIGNAL(repaintLayout()),
            gbs, SIGNAL(sigRepaintLayout()));
    connect(this, SIGNAL(findElement(const QString&, int, int)),
            gbs, SLOT(slotEditFind(const QString&, int, int)));
    connect(this, SIGNAL(sendFBChangeLayout(unsigned int, unsigned int,
                    bool)),
            gbs, SIGNAL(feedbackPortChanged(unsigned int, unsigned int,
                    bool)));
    connect(this, SIGNAL(sendFBBiDiChangeLayout(unsigned int, unsigned int,
                    bool, unsigned int)),
            gbs, SIGNAL(feedbackPortChanged(unsigned int, unsigned int,
                    bool, unsigned int)));
    connect(this, SIGNAL(switchedVisualMode(elemVisualMode)),
            gbs, SLOT(switchVisualMode(elemVisualMode)));
    connect(gbs, SIGNAL(sendSrcpMessage(SrcpMessage*)),
            this, SLOT(sendSrcpMessage(SrcpMessage*)));
    connect(gbs, SIGNAL(sigShowFBmodules()),
            this, SLOT(slotShowModules()));

    /*history line*/
    messageHistory = new MessageHistory(vBox);
    Q_CHECK_PTR(messageHistory);
    layout->addWidget(messageHistory);

    setCentralWidget(vBox);

    connect(commandPort, SIGNAL(messageReceived(const QString&)),
            messageHistory, SLOT(addCommand(const QString&)));
    connect(commandPort, SIGNAL(messageSend(const QString&)),
            messageHistory, SLOT(addCommand(const QString&)));
    connect(commandPort, SIGNAL(statusMessage(const QString&)),
            messageHistory, SLOT(addHint(const QString&)));

    connect(infoPort, SIGNAL(messageReceived(const QString&)),
            messageHistory, SLOT(addInfo(const QString&)));
    connect(infoPort, SIGNAL(messageSend(const QString&)),
            messageHistory, SLOT(addInfo(const QString&)));
    connect(infoPort, SIGNAL(statusMessage(const QString&)),
            messageHistory, SLOT(addHint(const QString&)));

    connect(this, SIGNAL(statusMessage(const QString&)),
            messageHistory, SLOT(addHint(const QString&)));
    connect(gbs, SIGNAL(statusMessage(const QString&)),
            messageHistory, SLOT(addHint(const QString&)));

    /*feedback module viewer*/
    fbViewer = new FeedbackViewer(this, "feedbackviewer");
    Q_CHECK_PTR(fbViewer);
    addDockWidget(Qt::LeftDockWidgetArea, fbViewer);
    connect(this, SIGNAL(sendFBChangeLayout(unsigned int, unsigned int,
                    bool)),
            fbViewer, SLOT(feedbackPortChanged(unsigned int, unsigned int,
                    bool)));
    fbViewer->hide();

    /*route controller*/
    router = new Router(this, gbs->getGbsElementListPtr(), "router");
    Q_CHECK_PTR(router);
    connect(this, SIGNAL(switchedVisualMode(elemVisualMode)),
            router, SLOT(switchVisualMode(elemVisualMode)));
    connect(this, SIGNAL(sendFBChangeRoute(unsigned int, unsigned int, bool)),
            router, SLOT(feedbackPortChanged(unsigned int,
                    unsigned int, bool)));
    connect(router, SIGNAL(sendSrcpMessage(SrcpMessage*)),
            this, SLOT(sendSrcpMessage(SrcpMessage*)));
    connect(gbs, SIGNAL(clearRoutes()),
            router, SLOT(clearRoutes()));
    connect(gbs, SIGNAL(recordElement(element*, elemRecordType)),
            router, SLOT(recordElement(element*, elemRecordType)));
    connect(gbs, SIGNAL(setRoute(element*, GbsButtonState,
                    GbsButtonState)),
            router, SLOT(setRoute(element*, GbsButtonState,
                    GbsButtonState)));
    connect(gbs, SIGNAL(resetRoute(element*, GbsButtonState)),
            router, SLOT(resetRoute(element*, GbsButtonState)));
    connect(router, SIGNAL(routeFunctionFinished()),
            gbs, SLOT(slotElementClickedTimeout()));
    connect(router, SIGNAL(startRouteTimer(Route::RouteType)),
            gbs, SLOT(startRouteTimer(Route::RouteType)));
    connect(gbs, SIGNAL(resetSelectedSignal()),
            router, SLOT(resetSelectedSignal()));
    connect(router, SIGNAL(updateRoutePathLEDs(const stateElement&,
                    const stateElement&, Route::RouteSetAction&)),
            gbs, SLOT(updateRoutePathLEDs(const stateElement&,
                    const stateElement&, Route::RouteSetAction&)));
    connect(router, SIGNAL(statusMessage(const QString&)),
            messageHistory, SLOT(addHint(const QString&)));

    /*route list window*/
    rtViewer = new RouteListWindow(this, "routeListWindow", router);
    addDockWidget(Qt::RightDockWidgetArea, rtViewer);
    rtViewer->hide();
    connect(this, SIGNAL(switchedVisualMode(elemVisualMode)),
            rtViewer, SLOT(switchVisualMode(elemVisualMode)));
    connect(rtViewer, SIGNAL(visibilityChanged(bool)),
            this, SLOT(updateRouteMenu(bool)));
    connect(rtViewer, SIGNAL(selectedRouteChangedState()),
            this, SLOT(updateRouteMenuActivateItems()));
    connect(rtViewer, SIGNAL(routeListIsEmpty()),
            this, SLOT(updateRouteListMenuItems()));
    connect(router, SIGNAL(routeListChanged()),
            rtViewer, SLOT(updateRouteList()));
    connect(router, SIGNAL(routeDataChanged(Route*)),
            rtViewer, SLOT(updateRouteData(Route*)));
    connect(router, SIGNAL(routeStateChanged(Route*)),
            rtViewer, SLOT(updateRouteState(Route*)));
    connect(router, SIGNAL(getElementByAddress(const int, const int,
                    element**)), gbs,
            SLOT(getElementByAddress(const int, const int, element**)));

    /*paint item window*/
    piw = new PaintItemWindow(this, "piw");
    Q_CHECK_PTR(piw);
    addDockWidget(Qt::LeftDockWidgetArea, piw);
    piw->hide();
    connect(piw, SIGNAL(paintItemChanged(SpdrPanel::SpdrItemClassId)),
            gbs, SLOT(changeLayoutPaintItem(SpdrPanel::SpdrItemClassId)));

    /*file toolbar*/
    QToolBar* filetb = new QToolBar(tr("File operations"), this);
    Q_CHECK_PTR(filetb);
    addToolBar(Qt::TopToolBarArea, filetb);

    /*file menu*/
    QMenu* filemenu = new QMenu(tr("&File"), this);
    menuBar()->addMenu(filemenu);

    /*file actions*/
    actionFileNew = new QAction(
            QIcon::fromTheme("document-new", QPixmap(filenew_xpm)),
            tr("&New..."), this);
    actionFileNew->setShortcut(QKeySequence::New);

    actionFileNew->setToolTip(tr("Create empty layout"));
    connect(actionFileNew, SIGNAL(triggered()), this,
            SLOT(slotFileNew()));
    filemenu->addAction(actionFileNew);
    filetb->addAction(actionFileNew);

    actionFileOpen = new QAction(
            QIcon::fromTheme("document-open", QPixmap(fileopen_xpm)),
            tr("&Open..."), this);
    actionFileOpen->setShortcut(QKeySequence::Open);
    actionFileOpen->setToolTip(tr("Open layout file"));
    connect(actionFileOpen, SIGNAL(triggered()), this,
            SLOT(fileOpen()));
    filemenu->addAction(actionFileOpen);
    filetb->addAction(actionFileOpen);

    actionFileOpenDemo = new QAction(
            QIcon::fromTheme("document-open", QPixmap(fileopen_xpm)),
            tr("&Open demo..."), this);
    //actionFileOpenDemo->setShortcut(QKeySequence::Open);
    actionFileOpenDemo->setToolTip(tr("Open demo layout file"));
    connect(actionFileOpenDemo, SIGNAL(triggered()), this,
            SLOT(fileOpenDemo()));
    filemenu->addAction(actionFileOpenDemo);
    //filetb->addAction(actionFileOpenDemo);

    fileRecentlyOpenedFiles = new QMenu(tr("&Recently opened files"), this);
    fileRecentlyOpenedFiles->setIcon(QIcon::fromTheme("document-open-recent"));
    filemenu->addMenu(fileRecentlyOpenedFiles);

    actionFileSave = new QAction(
            QIcon::fromTheme("document-save", QPixmap(filesave_xpm)),
            tr("&Save"), this);
    actionFileSave->setShortcut(QKeySequence::Save);
    connect(actionFileSave, SIGNAL(triggered()), this,
            SLOT(slotFileSave()));
    filemenu->addAction(actionFileSave);
    filetb->addAction(actionFileSave);

    actionFileSaveAs = new QAction(
            QIcon::fromTheme("document-save-as", QPixmap(filesaveas_xpm)),
            tr("Save &as..."), this);
    actionFileSaveAs->setShortcut(Qt::CTRL + Qt::Key_A);
    connect(actionFileSaveAs, SIGNAL(triggered()), this,
            SLOT(slotFileSaveAs()));
    filemenu->addAction(actionFileSaveAs);

    filemenu->addSeparator();
    //filetb->addSeparator();

    actionFileNewWindow = new QAction(
             QIcon::fromTheme("window-new", QPixmap(filenewwindow_xpm)),
            tr("New &window"), this);
    connect(actionFileNewWindow, SIGNAL(triggered()), this,
            SLOT(slotFileNewWin()));
    filemenu->addAction(actionFileNewWindow);

    if (firstinstance) {
        actionFileQuit = new QAction(
                QIcon::fromTheme("window-close", QPixmap(filequit_xpm)),
                tr("&Quit"), this);
        actionFileQuit->setShortcut(Qt::CTRL + Qt::Key_Q);
        connect(actionFileQuit, SIGNAL(triggered()), qApp,
                SLOT(closeAllWindows()));
        filemenu->addAction(actionFileQuit);
        firstinstance = false;
    }
    else {
        actionFileClose = new QAction(
                QIcon::fromTheme("window-close", QPixmap(fileclose_xpm)),
                tr("&Close"), this);
        actionFileClose->setShortcut(QKeySequence::Close);
        connect(actionFileClose, SIGNAL(triggered()), this,
                SLOT(close()));
        filemenu->addAction(actionFileClose);
    }

    // when the last window is closed, the application should quit
    connect(qApp, SIGNAL(lastWindowClosed()), qApp, SLOT(quit()));

    connect(filemenu, SIGNAL(aboutToShow()), this,
            SLOT(setupRecentFilesMenu()));
    connect(fileRecentlyOpenedFiles, SIGNAL(triggered(QAction*)), this,
            SLOT(recentFileActivated(QAction*)));

    /*edit toolbar*/
    QToolBar* edittb = new QToolBar(tr("Edit operations"), this);
    Q_CHECK_PTR(edittb);
    addToolBar(Qt::TopToolBarArea, edittb);

    QMenu* editfilemenu = new QMenu(tr("&Data files"), this);

    actionEditFileLayout = new QAction(tr("&Layout"), this);
    actionEditFileLayout->setToolTip(tr(
                "Edit layout file with external editor"));
    connect(actionEditFileLayout, SIGNAL(triggered()), this,
            SLOT(slotEditLayoutFile()));
    editfilemenu->addAction(actionEditFileLayout);

    actionEditFileOptions = new QAction(
            QDir::homePath() + "/" + SPDRS60_INIT, this);
    actionEditFileOptions->setToolTip(tr(
                "Edit config file with external editor"));
    connect(actionEditFileOptions, SIGNAL(triggered()), this,
            SLOT(slotEditConfigFile()));
    editfilemenu->addAction(actionEditFileOptions);


    /*edit menu*/
    QMenu* editmenu = new QMenu(tr("&Edit"), this);
    menuBar()->addMenu(editmenu);

    actionEditCut = new QAction(
            QIcon::fromTheme("edit-cut", QPixmap(editcut_xpm)),
            tr("Cu&t"), this);
    actionEditCut->setShortcut(QKeySequence::Cut);
    actionEditCut->setToolTip(tr("Cut selection to clipboard"));
    connect(actionEditCut, SIGNAL(triggered()), this,
            SLOT(slotEditCut()));
    editmenu->addAction(actionEditCut);
    edittb->addAction(actionEditCut);
    actionEditCut->setEnabled(false);

    actionEditCopy = new QAction(
            QIcon::fromTheme("edit-copy", QPixmap(editcopy_xpm)),
            tr("&Copy"), this);
    actionEditCopy->setShortcut(QKeySequence::Copy);
    actionEditCopy->setToolTip(tr("Copy selection to clipboard"));
    connect(actionEditCopy, SIGNAL(triggered()), this,
            SLOT(slotEditCopy()));
    editmenu->addAction(actionEditCopy);
    edittb->addAction(actionEditCopy);
    actionEditCopy->setEnabled(false);

    actionEditPaste = new QAction(
            QIcon::fromTheme("edit-paste", QPixmap(editpaste_xpm)),
            tr("&Paste"), this);
    actionEditPaste->setShortcut(QKeySequence::Paste);
    actionEditPaste->setToolTip(tr("Paste from clipboard"));
    connect(actionEditPaste, SIGNAL(triggered()), this,
            SLOT(slotEditPaste()));
    editmenu->addAction(actionEditPaste);
    edittb->addAction(actionEditPaste);
    actionEditPaste->setEnabled(false);

    editmenu->addSeparator();
    editmenu->addMenu(editfilemenu);

    editmenu->addSeparator();
    edittb->addSeparator();

    actionEditFind = new QAction(
            QIcon::fromTheme("edit-find", QPixmap(editfind_xpm)),
            tr("&Find..."), this);
    actionEditFind->setShortcut(QKeySequence::Find);
    actionEditFind->setToolTip(tr("Find information in layout element"));
    connect(actionEditFind, SIGNAL(triggered()), this,
            SLOT(slotEditFind()));
    editmenu->addAction(actionEditFind);
    edittb->addAction(actionEditFind);

    actionEditOptions = new QAction(QPixmap(editoptions_xpm),
            tr("Pr&eferences..."), this);
    actionEditOptions->setShortcut(Qt::CTRL + Qt::Key_P);
    actionEditOptions->setToolTip(tr("Edit application preferences"));
    connect(actionEditOptions, SIGNAL(triggered()), this,
            SLOT(slotEditOptions()));
    editmenu->addAction(actionEditOptions);


    /*view toolbar*/
    QToolBar* viewtb = new QToolBar(tr("View operations"), this);
    Q_CHECK_PTR(viewtb);
    addToolBar(Qt::TopToolBarArea, viewtb);

    QMenu* viewmenu = new QMenu(tr("&View"), this);
    menuBar()->addMenu(viewmenu);

    actionViewFBModules = new QAction(QPixmap(viewfeedback_xpm),
            tr("&Feedback modules"), this);
    actionViewFBModules->setShortcut(Qt::CTRL + Qt::Key_M);
    actionViewFBModules->setToolTip(tr("Show feedback module window"));
    connect(actionViewFBModules, SIGNAL(triggered()), this,
            SLOT(slotShowModules()));
    viewmenu->addAction(actionViewFBModules);
    viewtb->addAction(actionViewFBModules);

    actionViewRoutes = new QAction(QPixmap(viewroute_xpm),
            tr("Rou&te list"), this);
    actionViewRoutes->setShortcut(Qt::CTRL + Qt::Key_R);
    actionViewRoutes->setToolTip(tr("Show route list"));
    connect(actionViewRoutes, SIGNAL(triggered()), this,
            SLOT(slotShowRoutes()));
    viewmenu->addAction(actionViewRoutes);
    viewtb->addAction(actionViewRoutes);

    actionViewClock = new QAction(QPixmap(viewclock_xpm),
            tr("&Central clock"), this);
    actionViewClock->setToolTip(tr("Show central clock"));
    connect(actionViewClock, SIGNAL(triggered()), this,
            SLOT(slotShowClock()));
    viewmenu->addAction(actionViewClock);
    viewtb->addAction(actionViewClock);

    actionViewTrainNumberDialog = new QAction(QPixmap(viewtrainnumber_xpm),
            tr("Tr&ain number dialog"), this);
    actionViewTrainNumberDialog->setToolTip(
            tr("Show train number input dialog"));
    connect(actionViewTrainNumberDialog, SIGNAL(triggered()), this,
            SLOT(slotViewTrainNumberDialog()));
    viewmenu->addAction(actionViewTrainNumberDialog);
    viewtb->addAction(actionViewTrainNumberDialog);

    actionViewKeyboard = new QAction(QPixmap(viewkeyboard_xpm),
            tr("&Keyboard"), this);
    actionViewKeyboard->setShortcut(Qt::CTRL + Qt::Key_K);
    actionViewKeyboard->setToolTip(tr("Show basic keyboard"));
    connect(actionViewKeyboard, SIGNAL(triggered()), this,
            SLOT(slotViewKeyboard()));
    viewmenu->addAction(actionViewKeyboard);
    viewtb->addAction(actionViewKeyboard);

    viewmenu->addSeparator();
    viewtb->addSeparator();

    /*Layout mode group*/
    QActionGroup *ViewGrp = new QActionGroup(this);

    actionViewNormalMode = new QAction(QPixmap(viewnormalmode_xpm),
            tr("&Normal mode"), ViewGrp);
    actionViewNormalMode->setShortcut(Qt::CTRL + Qt::Key_L);
    actionViewNormalMode->setCheckable(true);

    actionViewLayoutEditMode = new QAction(QPixmap(viewlayouteditmode_xpm),
            tr("&Layout edit mode"), ViewGrp);
    actionViewLayoutEditMode->setShortcut(Qt::CTRL + Qt::Key_E);
    actionViewLayoutEditMode->setCheckable(true);

    actionViewRouteEditMode = new QAction(QPixmap(viewrouteeditmode_xpm),
            tr("&Route edit mode"), ViewGrp);
    actionViewRouteEditMode->setShortcut(Qt::CTRL + Qt::Key_B);
    actionViewRouteEditMode->setCheckable(true);

    actionViewNormalMode->setChecked(true);

    viewmenu->addActions(ViewGrp->actions());
    viewtb->addActions(ViewGrp->actions());

    connect(ViewGrp, SIGNAL(triggered(QAction*)), this,
            SLOT(slotViewSwitchMode(QAction*)));

    viewmenu->addSeparator();

    actionViewMenu = new QAction(tr("&Menu"), this);
    actionViewMenu->setShortcut(Qt::CTRL + Qt::Key_G);
    actionViewMenu->setToolTip(tr("Show the main menu"));
    actionViewMenu->setCheckable(true);
    actionViewMenu->setChecked(true);
    viewmenu->addAction(actionViewMenu);
    /*add action to mainwindow to keep responsiveness if menu is hidden*/
    addAction(actionViewMenu);
    connect(actionViewMenu, SIGNAL(toggled(bool)), menuBar(),
            SLOT(setVisible(bool)));

    actionViewToolbar = new QAction(
            tr("Tool&bars"), this);
    actionViewToolbar->setToolTip(tr("Show the toolbars"));
    actionViewToolbar->setShortcut(Qt::CTRL + Qt::Key_I);
    actionViewToolbar->setCheckable(true);
    actionViewToolbar->setChecked(true);
    viewmenu->addAction(actionViewToolbar);
    /*add action to mainwindow to keep responsiveness if menu is hidden*/
    addAction(actionViewToolbar);

    QAction* actionViewStatusline = new QAction(
            tr("&Statusline"), this);
    actionViewStatusline->setToolTip(tr("Show the statusline"));
    actionViewStatusline->setShortcut(Qt::CTRL + Qt::Key_J);
    actionViewStatusline->setCheckable(true);
    actionViewStatusline->setChecked(true);
    connect(actionViewStatusline, SIGNAL(toggled(bool)), messageHistory,
            SLOT(setVisible(bool)));
    viewmenu->addAction(actionViewStatusline);
    /*add action to mainwindow to keep responsiveness if menu is hidden*/
    addAction(actionViewStatusline);

    actionViewToggleFullScreen = new
        QAction(QIcon::fromTheme("view-fullscreen"),
            tr("Toggle f&ull screen"), this);
    actionViewToggleFullScreen->setShortcut(Qt::Key_F11);
    actionViewToggleFullScreen->setToolTip(tr("Toggle full screen mode"));
    connect(actionViewToggleFullScreen, SIGNAL(triggered()),
           this, SLOT(slotViewToggleFullscreen()));
    viewmenu->addAction(actionViewToggleFullScreen);
    /*add action to mainwindow to keep responsiveness if menu is hidden*/
    addAction(actionViewToggleFullScreen);

    viewmenu->addSeparator();

    actionViewToggleHistory = new QAction(
            tr("Toggle &history line"), this);
    actionViewToggleHistory->setShortcut(Qt::CTRL + Qt::Key_H);
    actionViewToggleHistory->setToolTip(tr("Toggle the history line"));
    connect(actionViewToggleHistory, SIGNAL(triggered()), messageHistory,
            SLOT(toggleLine()));
    viewmenu->addAction(actionViewToggleHistory);


    /*daemon toolbar*/
    daemontb = new QToolBar(tr("SRCP server operations"), this);
    Q_CHECK_PTR(daemontb);
    addToolBar(Qt::TopToolBarArea, daemontb);
    connect(actionViewNormalMode, SIGNAL(toggled(bool)),
            daemontb, SLOT(setVisible(bool)));

    QMenu* daemonmenu = new QMenu(tr("&SRCP server"), this);
    menuBar()->addMenu(daemonmenu);

    actionDaemonConnect = new QAction(QPixmap(daemonconnect_xpm),
            tr("&Connect"), this); // Ctrl D
    actionDaemonConnect->setToolTip(tr("Connect to SRCP server"));
    connect(actionDaemonConnect, SIGNAL(triggered()), this,
            SLOT(connectToSrcpServer()));
    daemonmenu->addAction(actionDaemonConnect);
    daemontb->addAction(actionDaemonConnect);

    actionDaemonDisconnect = new QAction(QPixmap(daemondisconnect_xpm),
            tr("&Disconnect"), this);
    actionDaemonDisconnect->setToolTip(tr("Disconnect from SRCP server"));
    connect(actionDaemonDisconnect, SIGNAL(triggered()), this,
            SLOT(closeSrcpServerConnection()));
    daemonmenu->addAction(actionDaemonDisconnect);
    daemontb->addAction(actionDaemonDisconnect);

    daemonmenu->addSeparator();

    actionDaemonReset = new QAction(QPixmap(daemonreset_xpm),
            tr("&Reset"), this);
    actionDaemonReset->setToolTip(tr("Reset SRCP server"));
    connect(actionDaemonReset, SIGNAL(triggered()), this,
            SLOT(slotDaemonReset()));
    daemonmenu->addAction(actionDaemonReset);

    actionDaemonKill = new QAction(QPixmap(daemonkill_xpm),
            tr("&Shutdown"), this);
    actionDaemonKill->setToolTip(tr("Shutdown SRCP server"));
    connect(actionDaemonKill, SIGNAL(triggered()), this,
            SLOT(slotDaemonKill()));
    daemonmenu->addAction(actionDaemonKill);

    actionDaemonInfo = new QAction(QPixmap(daemoninfo_xpm),
            tr("&Info..."), this);
    actionDaemonInfo->setToolTip(tr("Info about SRCP daemon"));
    connect(actionDaemonInfo, SIGNAL(triggered()), this,
            SLOT(slotDaemonInfo()));
    daemonmenu->addAction(actionDaemonInfo);
    daemontb->addAction(actionDaemonInfo);


    /*power toolbar*/
    powertb = new QToolBar(tr("Layout operations"), this);
    Q_CHECK_PTR(powertb);
    addToolBar(Qt::TopToolBarArea, powertb);
    connect(actionViewNormalMode, SIGNAL(toggled(bool)),
            powertb, SLOT(setVisible(bool)));

    /*layout toolbar*/
    layouttb = new QToolBar(tr("Layout operations"), this);
    Q_CHECK_PTR(layouttb);
    addToolBar(Qt::TopToolBarArea, layouttb);
    connect(actionViewNormalMode, SIGNAL(toggled(bool)),
            layouttb, SLOT(setVisible(bool)));

    QMenu* layoutmenu = new QMenu(tr("&Layout"), this);
    menuBar()->addMenu(layoutmenu);

    //TODO: create submenu to add separate items for each bus
    layoutSwitchBusPower = new QMenu(tr("&Switch layout power"), this);
    //layoutSwitchBusPower->setIcon(QIcon::fromTheme("document-open-recent"));
    layoutmenu->addMenu(layoutSwitchBusPower);
    layoutSwitchBusPower->setEnabled(false);

    connect(layoutSwitchBusPower, SIGNAL(triggered(QAction*)), this,
            SLOT(switchBusPower(QAction*)));

    actionLayoutFht = new QAction(tr("Use &FHT"), this);
    actionLayoutFht->setShortcut(Qt::Key_F5);
    actionLayoutFht->setToolTip(tr("Use route help button"));
    connect(actionLayoutFht, SIGNAL(triggered()), gbs,
            SLOT(slotFHTclicked()));
    layoutmenu->addAction(actionLayoutFht);

    actionLayoutUfgt = new QAction(tr("Use &UfGT"), this);
    actionLayoutUfgt->setShortcut(Qt::Key_F6);
    actionLayoutUfgt->setToolTip(tr("Use detour group button"));
    connect(actionLayoutUfgt, SIGNAL(triggered()), gbs,
            SLOT(slotUfGTclicked()));
    layoutmenu->addAction(actionLayoutUfgt);

    actionLayoutWgt = new QAction(tr("Use &WGT"),this);
    actionLayoutWgt->setShortcut(Qt::Key_F7);
    actionLayoutWgt->setToolTip(tr("Use turnout group button"));
    connect(actionLayoutWgt, SIGNAL(triggered()), gbs,
            SLOT(slotWGTclicked()));
    layoutmenu->addAction(actionLayoutWgt);

    actionLayoutSgt = new QAction(tr("Use &SGT"), this);
    actionLayoutSgt->setShortcut(Qt::Key_F8);
    actionLayoutSgt->setToolTip(tr("Use signal group button"));
    connect(actionLayoutSgt, SIGNAL(triggered()), gbs,
            SLOT(slotSGTclicked()));
    layoutmenu->addAction(actionLayoutSgt);

    actionLayoutHagt = new QAction(tr("Use H&aGT"), this);
    actionLayoutHagt->setShortcut(Qt::Key_F9);
    actionLayoutHagt->setToolTip(tr("Use signal halt group button"));
    connect(actionLayoutHagt, SIGNAL(triggered()), gbs,
            SLOT(slotHaGTclicked()));
    layoutmenu->addAction(actionLayoutHagt);

    layoutmenu->addSeparator();

    actionLayoutNotRot = new QAction(QPixmap(layoutnotrot_xpm),
            tr("&Halt signals"), this);
    actionLayoutNotRot->setShortcut(Qt::Key_F12);
    actionLayoutNotRot->setToolTip(tr("Switch all signals to halt"));
    connect(actionLayoutNotRot, SIGNAL(triggered()), gbs,
            SLOT(slotNotrot()));
    layoutmenu->addAction(actionLayoutNotRot);
    layouttb->addAction(actionLayoutNotRot);

    actionLayoutToggleAll = new QAction(tr("&Toggle all"), this);
    actionLayoutToggleAll->setShortcut(Qt::CTRL + Qt::Key_F10);
    actionLayoutToggleAll->setToolTip(tr("Toggle all switchable elements"));
    connect(actionLayoutToggleAll, SIGNAL(triggered()), gbs,
            SLOT(slotToggleAll()));
    layoutmenu->addAction(actionLayoutToggleAll);

    actionLayoutSendAll = new QAction(tr("Send &all"), this);
    actionLayoutSendAll->setShortcut(Qt::CTRL + Qt::Key_F11);
    actionLayoutSendAll->setToolTip(tr("Send current states of all "
                "switchable elements to SRCP server"));
    connect(actionLayoutSendAll, SIGNAL(triggered()), this,
            SLOT(layoutSendAll()));
    layoutmenu->addAction(actionLayoutSendAll);

    layoutmenu->addSeparator();

    actionLayoutChangeSize = new QAction(
            tr("S&ettings..."), this);
    actionLayoutChangeSize->setToolTip(tr("Change layout settings"));
    connect(actionLayoutChangeSize, SIGNAL(triggered()), this,
            SLOT(layoutChangeSize()));
    layoutmenu->addAction(actionLayoutChangeSize);


    /*route toolbar*/
    QToolBar* routetb = new QToolBar(tr("Route operations"), this);
    Q_CHECK_PTR(routetb);
    addToolBar(Qt::TopToolBarArea, routetb);
    routetb->hide();
    //connect visibility of toolbar to visibility of routingviewer
    connect(rtViewer, SIGNAL(visibilityChanged(bool)),
               routetb, SLOT(setVisible(bool)));

    /*Route menu*/
    QMenu* routemenu = new QMenu(tr("&Route"), this);
    menuBar()->addMenu(routemenu);

    actionRouteActivate = new QAction(QPixmap(route_start_xpm),
            tr("Ac&tivate"), this);
    actionRouteActivate->setToolTip(tr("Activate route"));
    actionRouteActivate->setShortcut(Qt::Key_Plus);
    connect(actionRouteActivate, SIGNAL(triggered()), rtViewer,
            SLOT(slotRouteActivate()));
    routemenu->addAction(actionRouteActivate);
    routetb->addAction(actionRouteActivate);

    actionRouteWithdraw = new QAction(QPixmap(route_stop_xpm), tr("W&ithdraw"),
            this);
    actionRouteWithdraw->setToolTip(tr("Withdraw a wrong activated route"));
    actionRouteWithdraw->setText(tr("Withdraw route"));
    actionRouteWithdraw->setShortcut(Qt::CTRL + Qt::Key_Minus);
    connect(actionRouteWithdraw, SIGNAL(triggered()), rtViewer,
            SLOT(slotRouteWithdraw()));
    routemenu->addAction(actionRouteWithdraw);
    routetb->addAction(actionRouteWithdraw);

    actionRouteRelease = new QAction(tr("&Release"), this);
    actionRouteRelease->setToolTip(tr("Release an activated route. This\n"
                "includes a train number transfer."));
    actionRouteRelease->setShortcut(Qt::Key_Minus);
    connect(actionRouteRelease, SIGNAL(triggered()), rtViewer,
            SLOT(slotRouteRelease()));
    routemenu->addAction(actionRouteRelease);

    routemenu->addSeparator();
    routetb->addSeparator();

    actionRouteAdd = new QAction(QPixmap(route_new_xpm), tr("&Add"), this);
    actionRouteAdd->setToolTip(tr("Add new route"));
    connect(actionRouteAdd, SIGNAL(triggered()), this,
            SLOT(slotRouteAdd()));
    routemenu->addAction(actionRouteAdd);
    routetb->addAction(actionRouteAdd);

    actionRouteEdit = new QAction(QPixmap(route_edit_xpm),
            tr("&Edit..."), this);
    actionRouteEdit->setToolTip(tr("Edit selected route"));
    connect(actionRouteEdit, SIGNAL(triggered()), rtViewer,
            SLOT(slotRouteEdit()));
    routemenu->addAction(actionRouteEdit);
    routetb->addAction(actionRouteEdit);

    actionRouteCopy = new QAction(QPixmap(route_copy_xpm),
            tr("Dupli&cate"), this);
    actionRouteCopy->setToolTip(tr("Duplicate selected route"));
    connect(actionRouteCopy, SIGNAL(triggered()), rtViewer,
            SLOT(slotRouteCopy()));
    routemenu->addAction(actionRouteCopy);
    routetb->addAction(actionRouteCopy);

    actionRouteDelete = new QAction(QPixmap(route_clear_xpm),
            tr("&Delete"), this);
    actionRouteDelete->setToolTip(tr("Delete selected route"));
    connect(actionRouteDelete, SIGNAL(triggered()), this,
            SLOT(slotRouteDelete()));
    routemenu->addAction(actionRouteDelete);
    routetb->addAction(actionRouteDelete);

    routemenu->addSeparator();

    actionRouteUnlockAll = new QAction(tr("&Unlock all"), this);
    actionRouteUnlockAll->setShortcut(Qt::CTRL + Qt::Key_U);
    actionRouteUnlockAll->setToolTip(tr("Unlock all routes"));
    connect(actionRouteUnlockAll, SIGNAL(triggered()), router,
            SLOT(unlockAllLockedRoutes()));
    routemenu->addAction(actionRouteUnlockAll);

    actionRouteSendTrainNumbers = new QAction(
            tr("&Send all train numbers"), this);
    //actionRouteSendTrainNumbers->setShortcut(Qt::CTRL + Qt::Key_T);
    actionRouteSendTrainNumbers->setToolTip(tr("Send all train numbers "
                "as CRCF message to other SRCP clients"));
    connect(actionRouteSendTrainNumbers, SIGNAL(triggered()), router,
            SLOT(sendTrainNumbers()));
    routemenu->addAction(actionRouteSendTrainNumbers);


    /*layout edit toolbar*/
    layoutedittb = new QToolBar(tr("Layout edit operations"), this);
    Q_CHECK_PTR(layoutedittb);
    addToolBar(Qt::TopToolBarArea, layoutedittb);
    layoutedittb->hide();
    layoutedittb->setEnabled(false);

    //connect visibility of toolbar to toggle state of layout edit mode

    connect(piw, SIGNAL(visibilityChanged(bool)),
            layoutedittb, SLOT(setVisible(bool)));

    //action
    connect(actionViewLayoutEditMode, SIGNAL(toggled(bool)),
            layoutedittb, SLOT(setVisible(bool)));
    connect(actionViewLayoutEditMode, SIGNAL(toggled(bool)),
            layoutedittb, SLOT(setEnabled(bool)));

    //connect visibility of paint item window
    connect(actionViewLayoutEditMode, SIGNAL(toggled(bool)),
            piw, SLOT(setVisible(bool)));

    LayoutEditModeAgrp *layoutEditGrp = new LayoutEditModeAgrp(this,
            "layoutEditModeGroup");
    for (int i = 0; i < layoutEditGrp->actions().size(); ++i) {
        layoutedittb->addAction(layoutEditGrp->actions().at(i));
    }

    connect(layoutEditGrp, SIGNAL(modeChanged(GBSArea::LayoutEditMode)),
            gbs, SLOT(changeLayoutEditMode(GBSArea::LayoutEditMode)));
    connect(layoutEditGrp, SIGNAL(modeChanged(GBSArea::LayoutEditMode)),
            piw, SLOT(changeLayoutEditMode(GBSArea::LayoutEditMode)));


    /*help toolbar*/
    //QToolBar* helptb = new QToolBar(tr("Help operations"), this);
    //addToolBar(Qt::TopToolBarArea, helptb);
    //Q_CHECK_PTR(helptb);

    QMenu* helpmenu = new QMenu(tr("&Help"), this);
    menuBar()->addMenu(helpmenu);

    helpmenu->addAction(tr("&Help"), this, SLOT(slotAboutHelp()), Qt::Key_F1);
    helpmenu->addAction(tr("&spdrs60 for Linux on the web"),
            this, SLOT(slotAboutWeb()));
    helpmenu->addSeparator();
    helpmenu->addAction(tr("&About %1...").arg(PACKAGE),
            this, SLOT(slotAbout()));
    helpmenu->addAction(tr("About &Qt..."), this, SLOT(slotAboutQt()));

    // connect all toolbars to be hidable
    connect(actionViewToolbar, SIGNAL(toggled(bool)), filetb,
            SLOT(setVisible(bool)));
    connect(actionViewToolbar, SIGNAL(toggled(bool)), edittb,
            SLOT(setVisible(bool)));
    connect(actionViewToolbar, SIGNAL(toggled(bool)), viewtb,
            SLOT(setVisible(bool)));
    connect(actionViewToolbar, SIGNAL(toggled(bool)), daemontb,
            SLOT(setVisible(bool)));
    connect(actionViewToolbar, SIGNAL(toggled(bool)), powertb,
            SLOT(setVisible(bool)));
    connect(actionViewToolbar, SIGNAL(toggled(bool)), layouttb,
            SLOT(setVisible(bool)));
    //connect(actionViewHideToolbar, SIGNAL(toggled(bool)), routetb,
    //        SLOT(setVisible(bool)));

    resetMenu(); //may be is obsolete
}


// resetMenu added by dirk. Before loading a new layout, the settings of
// the old one must be reset.
void MainWindow::resetMenu()
{
    // always false on setup, a layout must be loaded first
    actionFileSave->setEnabled(false);
    actionFileSaveAs->setEnabled(false);
    actionEditFileLayout->setEnabled(false);
    actionEditFind->setEnabled(false);
    actionViewRoutes->setEnabled(false);
    actionViewLayoutEditMode->setEnabled(false);
    actionViewRouteEditMode->setEnabled(false);
    // setup depends on daemonstartup
    updateDaemonMenu();
    // always false on setup, a layout must be loaded first
    actionLayoutFht->setEnabled(false);
    actionLayoutWgt->setEnabled(false);
    actionLayoutUfgt->setEnabled(false);
    actionLayoutSgt->setEnabled(false);
    actionLayoutHagt->setEnabled(false);
    actionLayoutNotRot->setEnabled(false);
    actionLayoutChangeSize->setEnabled(false);
    actionRouteUnlockAll->setEnabled(false);
}


void MainWindow::slotEditCut()
{
 /*TODO*/
}


void MainWindow::slotEditCopy()
{
 /*TODO*/
}


void MainWindow::slotEditPaste()
{
 /*TODO*/
}


void MainWindow::readAutoloadFile()
{
    if (!pref.autoload)
        return;

    // if autoload file from config data does
    // not exist ask user to change options

    if (!QFile::exists(pref.autolayout)) {
        qApp->beep();
        int choice = QMessageBox::warning(this, tr("Autoloader failed"),
                         tr("The selected autoload file '%1'\n"
                            "does not exist. Please adjust your"
                            " options.").arg(pref.autolayout),
                         tr("&Now"), tr("&Later"), 0, 0, 0);
        if (choice == 0)
            slotEditOptions();
    }
    else
        openFile(pref.autolayout);
}


void MainWindow::closeEvent(QCloseEvent* e)
{
    if (pref.autosave) {
        if (saveFile())
            e->accept();
        else
            e->ignore();
        return;
    }

    if (isSave())
        e->accept();
    else
        e->ignore();
}


int MainWindow::querySaveChanges()
{
    QString queryStr;

    if (fileName.isEmpty())
        queryStr = tr("Unnamed file was changed.\nSave changes?");
    else
        queryStr = tr("File '%1' was changed.\n"
                "Save changes?").arg(fileName);

    return QMessageBox::warning(this, tr("Save changes"),
            queryStr, QMessageBox::Yes | QMessageBox::No |
            QMessageBox::Cancel);
}


void MainWindow::slotFileNew()
{
    if (isSave())
        newFile();
}


void MainWindow::newFile()
{
    // show a dialog where the user can put in layout dimensions
    newLayoutDialog* nlDlg = new newLayoutDialog(this);
    if (nlDlg == NULL)
        return;

    nlDlg->setColumns(pref.layoutcols);
    nlDlg->setRows(pref.layoutrows);
    nlDlg->setHost(commandPort->getHostname());
    nlDlg->setPort(commandPort->getPortNumber());
    nlDlg->setAutoLogin(cmdAutoLogin);
    nlDlg->setAutoPower(cmdAutoPower);
    nlDlg->setAutoSendAll(cmdAutoSendAll);

    nlDlg->setSwitchboxId(rwccid);
    nlDlg->setSwitchboxName(rwccname);
    nlDlg->setLayoutId(gbs->getLayoutId());
    nlDlg->setLayoutName(gbs->getLayoutName());

    if (nlDlg->exec() != QDialog::Accepted) {
        delete nlDlg;
        return;
    }
    closeSrcpServerConnection();

    int newcols = nlDlg->getColumns();
    int newrows = nlDlg->getRows();
    unsigned int newid = nlDlg->getLayoutId();
    QString newname = nlDlg->getLayoutName();
    commandPort->setServer(nlDlg->getHost(), nlDlg->getPort());
    infoPort->setServer(nlDlg->getHost(), nlDlg->getPort());

    cmdAutoLogin = nlDlg->getAutoLogin();
    cmdAutoPower = nlDlg->getAutoPower();
    cmdAutoSendAll = nlDlg->getAutoSendAll();

    rwccid = nlDlg->getSwitchboxId();
    rwccname = nlDlg->getSwitchboxName();

    delete nlDlg;

    fileName = "";
    gbs->newFile(newcols, newrows, newid, newname);

    updateCaption();
    updateFileMenuItems();
    statusMessage(tr("New layout file created"));
}


void MainWindow::updateFileMenuItems()
{
    actionFileSave->setEnabled(true);
    actionFileSaveAs->setEnabled(true);

    if (fileName.isEmpty()) {
        actionEditFileLayout->setText(tr("Layout file not saved yet"));
        actionEditFileLayout->setEnabled(false);
    }
    else {
        actionEditFileLayout->setText(fileName);
        actionEditFileLayout->setEnabled(true);
    }
    actionEditFind->setEnabled(true);

    actionLayoutFht->setEnabled(true);
    actionLayoutWgt->setEnabled(true);
    actionLayoutUfgt->setEnabled(true);
    actionLayoutSgt->setEnabled(true);
    actionLayoutHagt->setEnabled(true);
    actionLayoutNotRot->setEnabled(true);
    actionLayoutChangeSize->setEnabled(true);

    bool connected = commandPort->hasServerConnection();
    actionLayoutToggleAll->setEnabled(connected);
    actionLayoutSendAll->setEnabled(connected);

    //viewmenu->setItemEnabled(VIEW_ID_ROUTES, !fileName.isEmpty());
    actionViewRoutes->setEnabled(true);
    actionViewLayoutEditMode->setEnabled(true);
    actionViewRouteEditMode->setEnabled(true);
    actionRouteUnlockAll->setEnabled(true);
}


bool MainWindow::saveFile()
{
    if (fileName.isEmpty()) {
        slotFileSaveAs();
        return true;
    }

    QFile f(fileName);
    if (!f.open(QIODevice::WriteOnly)) {
        statusMessage(tr("Could not write to file '%1'").arg(fileName));
        return false;
    }

    QTextStream ts(&f);
    ts.setCodec("UTF-8");
    QDateTime dt = QDateTime::currentDateTime();

    // write the header
    ts << "# spdrs60 data file" << endl
       << "# version=" << VERSION << endl
       << "# last modified=" << dt.toString(Qt::ISODate) << endl
       << GF_FORMATVERSION << DS << GF_FV << endl
       << GF_CMDHOST << DS << commandPort->getHostname()
       << DS << commandPort->getPortNumber()
       << DS << cmdAutoLogin << DS << cmdAutoPower
       << DS << cmdAutoSendAll << endl
       << GF_RWCC << DS << rwccid << DS << rwccname << endl;

    gbs->writeFileTextToStream(ts);
    router->writeFileTextToStream(ts);

    f.close();

    updateCaption();
    updateFileMenuItems();

    statusMessage(tr("Layout file '%1' saved").arg(fileName));
    return true;
}


void MainWindow::slotFileSave()
{
    saveFile();
}


void MainWindow::slotFileSaveAs()
{
    QString fn = QFileDialog::getSaveFileName(this,
            tr("Save layout file"), lastDir,
            QString(tr("Layouts")) + " (*" + GF_GBSEXT + ")");

    if (!fn.isEmpty()) {

        lastDir = fn.left(fn.lastIndexOf('/'));

        /*check for file extension*/
        if (!fn.endsWith(GF_GBSEXT))
            fn.append(GF_GBSEXT);

        fileName = fn;
        saveFile();
    }
    else
        statusMessage(tr("Saving aborted"));
}


bool MainWindow::isSave()
{
    bool result = false;

    if (isModified()) {
        int choice = querySaveChanges();
        switch (choice) {
            case QMessageBox::Yes:
                if (saveFile())
                    result = true;
                break;
            case QMessageBox::No:
                result = true;
                break;
            case QMessageBox::Cancel:
            default:
                break;
        }
    }
    else
        result = true;
    return result;
}


void MainWindow::fileOpen()
{
    if (isSave())
        chooseFile();
}


void MainWindow::fileOpenDemo()
{
    if (isSave())
        chooseDemoFile();
}


void MainWindow::chooseFile()
{
    QStringList fnl = QFileDialog::getOpenFileNames(this,
            tr("Open layout file"), lastDir,
        QString(tr("Layouts")) + " (*" + GF_GBSEXT + ")");

    if (fnl.isEmpty())
        return;
    else {
        openFile(fnl.first());
        fnl.pop_front();

        QStringList::Iterator it = fnl.begin();
        while (it != fnl.end()) {
                openFileWindow(*it);
                ++it;
        }
    }
}


void MainWindow::chooseDemoFile()
{
    QStringList fnl = QFileDialog::getOpenFileNames(this,
            tr("Open demo layout file"), DEMOFILEPATH,
        QString(tr("Layouts")) + " (*" + GF_GBSEXT + ")");

    if (fnl.isEmpty())
        return;
    else {
        openFile(fnl.first());
        fnl.pop_front();

        QStringList::Iterator it = fnl.begin();
        while (it != fnl.end()) {
                openFileWindow(*it);
                ++it;
        }
    }
}


void MainWindow::openFile(const QString& fn)
{
    /*remember last directory we used*/
    lastDir = fn.left(fn.lastIndexOf('/'));

    QFile f(fn);

    if (!f.open(QIODevice::ReadOnly)) {
        statusMessage(tr("Could not read file '%1'").arg(fn));
        return;
    }
    closeSrcpServerConnection();

    // give application some time to handle socket closing and toolbar
    // painting
    qApp->processEvents();

    fileName = fn;
    addRecentlyOpenedFile(fn, recentFiles);
    QTextStream ts(&f);
    ts.setCodec("UTF-8");

    QString s, key, value;
    //int fversion = 0;

    while (!ts.atEnd()) {
        s = ts.readLine();

        /* ignore comment lines */
        if (!s.startsWith("#")) {
            key = s.section(DS, 0, 0);
            value = s.section(DS, 1, 1).trimmed();

            /* key/value pairs are read sequence independent */
            if (key.compare(GF_CMDHOST) == 0) {
                QStringList tokens = s.split(DS);

                unsigned int prt = tokens[2].toUInt();
                commandPort->setServer(tokens[1], prt);
                infoPort->setServer(tokens[1], prt);

                cmdAutoLogin = tokens[3].toInt() == 1;
                cmdAutoPower = tokens[4].toInt() == 1;
                cmdAutoSendAll = tokens[5].toInt() == 1;
            }
            //else if (key.compare(GF_FORMATVERSION) == 0)
            //    fversion = value.toInt();

            else if (key.compare(GF_RWCC) == 0) {
                rwccid = value.toUInt();
                rwccname = s.section(DS, 2);
            }

            else if (s.startsWith("%% layout"))
                break;
        }
    }

    gbs->readFileTextFromStream(ts);
    qApp->processEvents();
    router->readFileTextFromStream(ts);

    f.close();

    statusMessage(tr("Layout file '%1' opened").arg(fn));
    updateCaption();
    updateFileMenuItems();

    if (cmdAutoLogin) {
        connectToSrcpServer();
        qApp->processEvents();
    }
}


void MainWindow::updateCaption()
{
    if (fileName.isEmpty())
        setWindowTitle(QString(PACKAGE) + " - [" + tr("noname") + "]");
    else
        setWindowTitle(QString(PACKAGE) + " - [" + fileName + "]");
}


void MainWindow::processCommandMessage(const SrcpMessage* sm)
{
    switch (sm->getType()) {
        case SrcpMessage::mtAckHandShake:
            //option to save session id
            break;
        case SrcpMessage::mtAckCommand:
            if (SRCPCommandState == srcp08TermServer) {
                if (SrcpMessage::code200 == sm->getCode())
                    SRCPCommandState = srcpUndefined;
            }
            else if (SRCPCommandState == srcp08InitGADevices)
                runGaInitSequence();
            else if (SRCPCommandState == srcp08SetBusPower)
                runSetBusPowerSequence();
            break;
        default:
            break;
    }
}

/*
 * respond to connect/disconnect changes
 *
 * in SRCP 0.8 mode connection is established to same port as
 * command channel, but other login type
 */
void MainWindow::updateCommandConnectionState(bool connected)
{
    if (connected) {

        statusMessage(tr("Command port connected."));
        infoPort->setServer(
                commandPort->getHostname(),
                commandPort->getPortNumber());
        infoPort->serverConnect();

        /*
         * now the server is ready for basic commands
         *
         * the complete server initialization will go through
         * this sequence:
         *   1) initialize all GAs
         *   3) power on all busses if automode is set
         *   4) Send all GA states if send state is set
         */

        SRCPCommandState = srcp08InitGADevices;
        runGaInitSequence();
        // layout area is set modified to make changes of GA directions
        // saveable
        gbs->setModified(true);
    }
    else
        SRCPCommandState = srcpUndefined;

    updateDaemonMenu();
}


/*
 * react to connect/disconnect changes
 */
void MainWindow::updateInfoConnectionState(bool connected)
{
    if (connected) {
        router->setInfoSessionId(infoPort->getSessionId());
        statusMessage(tr("Info port connected."));
    }
    else
        router->setInfoSessionId(0);
}


void MainWindow::handleDescriptionInfoMessage(const DescriptionMessage* dm)
{
    if (SrcpMessage::code100 == dm->getCode()) {
        if (0 == dm->getBus()){
            router->setServerHasGm(dm->hasGm());
        }
        else if (dm->hasPower()) {
            // add power switch item for each bus
            addBusPowerItem(dm->getBus());
        }
    }
}


void MainWindow::handleFbInfoMessage(const FbMessage* fbm)
{
    switch (fbm->getCode()) {
        case SrcpMessage::code100:
            // send updates to:
            // 1. module window
            // 2. all elements via gbs
            // 3. all routes if not in init mode
            /*emit sendFBChangeModule(
                    fbm->getBus(),
                    fbm->getAddress(),
                    fbm->getValue());*/

            emit sendFBChangeLayout(
                    fbm->getBus(),
                    fbm->getAddress(),
                    1 == fbm->getValue());

            emit sendFBChangeRoute(
                    fbm->getBus(),
                    fbm->getAddress(),
                    1 == fbm->getValue());
            break;
        case SrcpMessage::code103:
            //BiDi GL message
            emit sendFBBiDiChangeLayout(
                    fbm->getBus(),
                    fbm->getAddress(),
                    true,
                    fbm->getGlAddress());
            break;
        default:
            break;
    }
}


void MainWindow::handleGaInfoMessage(const GaMessage* gam)
{
    if (SrcpMessage::code100 == gam->getCode()) {
        // (bus, addr, port, value)
        gbs->sendInfoPortMessage(
                gam->getBus(),
                gam->getAddress(),
                gam->getPort(),
                gam->getValue());
    }
}


void MainWindow::handleGmInfoMessage(const GmMessage* gmm)
{
    unsigned int sendid = gmm->getSendId();
    unsigned int replyid = gmm->getReplyId();

    //shortcut not to reply to own messages
    if (infoPort->getSessionId() == replyid)
        return;

    // we are not the addressee
    if (sendid != 0 && infoPort->getSessionId() != sendid)
        return;

    CrcfMessage* cm = gmm->createCrcfMessage();
    if (NULL == cm) {
        statusMessage(tr("Unvalid CRCF message received"));
        return;
    }

    switch (cm->getActor()) {
        case CrcfMessage::acTrain:
            router->handleCrcfTrainMessage(replyid, cm);
            break;
        case CrcfMessage::acRoute:
            router->processGenericMessage(replyid, sendid, cm);
            break;
        case CrcfMessage::acLayout:
            gbs->processGenericMessage(replyid, sendid, cm);
            break;
        case CrcfMessage::acRwcc:
            processGenericMessage(replyid, sendid, cm);
            break;
        default:
            statusMessage(tr("Unsupported CRCF actor detected: %1")
                    .arg(cm->getActor()));
            break;
    }
    delete cm;
}


void MainWindow::handlePowerInfoMessage(const PowerMessage* pm)
{
    if (SrcpMessage::code100 == pm->getCode()) {
        /*Update qaction/menu item to display current power state*/
        for (int i = 0; i < layoutSwitchBusPower->actions().count();
                i++) {
            QAction * ac = layoutSwitchBusPower->actions()[i];
            if (pm->getBus() == ac->data().toUInt()) {
                bool ison = pm->ison();
                if (ison)
                    ac->setIcon(QPixmap(layoutstop_xpm));
                else
                    ac->setIcon(QPixmap(layoutstart_xpm));
                ac->setChecked(ison);
            }
        }
    }
}


/* respond to incomming info messages */
void MainWindow::processInfoMessage(const SrcpMessage* sm)
{
    switch (sm->getType()) {
        case SrcpMessage::mtAckHandShake:
            // possibility to save session id
            break;
        case SrcpMessage::mtDescriptionInfo:
            handleDescriptionInfoMessage(((DescriptionMessage*)sm));
            break;
        case SrcpMessage::mtFbInfo:
            handleFbInfoMessage(((FbMessage*)sm));
            break;
        case SrcpMessage::mtGaInfo:
            handleGaInfoMessage(((GaMessage*)sm));
            break;
        case SrcpMessage::mtGmInfo:
            handleGmInfoMessage(((GmMessage*)sm));
            break;
        case SrcpMessage::mtPowerInfo:
            handlePowerInfoMessage(((PowerMessage*)sm));
            break;
        case SrcpMessage::mtServerInfo:
            break;
        default:
            break;
    }
}


void MainWindow::connectToSrcpServer()
{
    ConnectCommandPort();
}

/*
 * connection to all server ports is done in a cascade, first try is
 * command port
 * also preset Generic Massage capabilities of server to "no"
 */
void MainWindow::ConnectCommandPort()
{
    router->setServerHasGm(false);
    commandPort->serverConnect();
}


void MainWindow::closeSrcpServerConnection()
{
    /* 1. Command socket */
    if (commandPort->hasServerConnection()) {

        ServerMessage sm = ServerMessage(SrcpMessage::mtSessionTerm);
        sendSrcpMessage(&sm);

        commandPort->serverDisconnect();
        SRCPCommandState = srcpUndefined;
    }

    /* 2. Info socket */
    if (infoPort->hasServerConnection()) {
        infoPort->serverDisconnect();
    }
}


void MainWindow::sendSrcpMessage(SrcpMessage* sm)
{
    if (sm == NULL)
        return;

    if (!commandPort->hasServerConnection())
        return;

    QString cmd = sm->getSrcpMessageStr();
    commandPort->sendToServer(cmd);
}
/* End of new Networking code */


// reset the daemon
void MainWindow::slotDaemonReset()
{
    if (QMessageBox::Yes == QMessageBox::warning(this, tr("Server reset"),
                tr("You are going to reset the SRCP server.\n"
                    "Are you sure?"),
                QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel))
    {
        ServerMessage sm = ServerMessage(SrcpMessage::mtServerReset);
        sendSrcpMessage(&sm);
    }
}


/*send shut down SRCP server message*/
void MainWindow::slotDaemonKill()
{
    if (QMessageBox::Yes == QMessageBox::warning(this,
                tr("Shutdown SRCP server"),
                tr("You are about to shutdown the SRCP server.\n"
                    "Do you really want to proceed?\n"
                    "(Note: To continue using this program, restart\n"
                    "the server daemon after shutdown has finished)"),
                QMessageBox::Yes | QMessageBox::No |
                QMessageBox::Cancel))
    {
        ServerMessage sm = ServerMessage(SrcpMessage::mtServerTerm);
        sendSrcpMessage(&sm);

        closeSrcpServerConnection();
        statusMessage(tr("SRCP-Service has been terminated. Restart "
                    "server to reconnect %1 for Linux").arg(PACKAGE));
    }
}


/*show SRCP server info window*/
void MainWindow::slotDaemonInfo()
{
    ServerInfoDialog* sid = new ServerInfoDialog(this,
            "serverInfoDialog");

    if (sid == NULL)
        return;

    if (commandPort->hasServerConnection())
        sid->setCommandSessionData(
                commandPort->getSrcpServer(),
                commandPort->getSrcpVersion(),
                commandPort->getSrcpOther(),
                commandPort->getSessionId(),
                router->serverHasGm());
    else
        sid->setCommandSessionData(tr("Not connected"), "", "", 0, false);

    if (infoPort->hasServerConnection())
        sid->setInfoSessionData(
                infoPort->getSrcpServer(),
                infoPort->getSrcpVersion(),
                infoPort->getSrcpOther(),
                infoPort->getSessionId());
    else
        sid->setInfoSessionData(tr("Not connected"), "", "", 0);

    sid->exec();
    delete sid;
}


/*update state of daemon/server menu items*/
void MainWindow::updateDaemonMenu()
{
    bool connected = commandPort->hasServerConnection();

    if (!connected) {
        /*remove bus power items*/
        powertb->clear();
        layoutSwitchBusPower->clear();
        layoutSwitchBusPower->setEnabled(false);
    }
    // disable all daemon related menus and toolbuttons if the daemon is
    // not running or has been killed
    actionViewKeyboard->setEnabled(connected);

    actionDaemonConnect->setEnabled(!connected);
    actionDaemonDisconnect->setEnabled(connected);
    actionDaemonReset->setEnabled(connected);
    actionDaemonKill->setEnabled(connected);
    actionDaemonInfo->setEnabled(connected);

    actionLayoutToggleAll->setEnabled(connected);
    actionLayoutSendAll->setEnabled(connected);
    actionRouteSendTrainNumbers->setEnabled(connected);
}


/*show spdrs60 copyright message window*/
void MainWindow::slotAbout()
{
    AboutDialog* ad = new AboutDialog(this);
    if (ad == NULL)
        return;

    ad->exec();
    delete ad;
}


/*show Qt copyright window*/
void MainWindow::slotAboutQt()
{
    QMessageBox::aboutQt(this, tr("About Qt"));
}


/*show/hide route list window*/
void MainWindow::slotShowRoutes()
{
    if (rtViewer!= NULL) {
        if (rtViewer->isVisible())
            rtViewer->hide();
        else {
            rtViewer->show();
            rtViewer->raise();
        }
    }
}


/* edit layout file with external editor*/
void MainWindow::slotEditLayoutFile()
{
    QStringList arguments;
    QString program;

    if (pref.editor.contains(' ')) {
        program = pref.editor.section(' ', 0, 0);
        arguments << pref.editor.section(' ', 1);
    }
    else
        program = pref.editor;

    arguments << fileName;

    QProcess* proc = new QProcess(this);
    if (proc != NULL)
        proc->start(program, arguments);
}


/*open config file with external editor*/
void MainWindow::slotEditConfigFile()
{
    QStringList arguments;
    QString program;

    if (pref.editor.contains(' ')) {
        program = pref.editor.section(' ', 0, 0);
        arguments << pref.editor.section(' ', 1);
    }
    else
        program = pref.editor;

    arguments << QDir::homePath() << QString("/").append(SPDRS60_INIT);

    QProcess* proc = new QProcess(this);
    if (proc != NULL)
        proc->start(program, arguments);
}


/*switch edit modes of layout area*/
void MainWindow::slotViewSwitchMode(QAction* ac)
{
    if ((ac == actionViewNormalMode) && (visualMode != kvmNormal))
        switchToNormalMode();

    else if ((ac == actionViewLayoutEditMode) && (visualMode != kvmEditLayout))
        switchToEditLayoutMode();

    else if ((ac == actionViewRouteEditMode) && (visualMode != kvmEditRoute))
        switchToEditRouteMode();

    else
        statusMessage(tr("Unsupported action detected: '%1'!").arg(ac->text()));
}


void MainWindow::propagateViewModeSwitch()
{
    // send new visual mode to router, gbs and route list window
    emit switchedVisualMode(visualMode);

    // change edit related menus
    actionFileNew->setEnabled(visualMode == kvmNormal);
    actionFileOpen->setEnabled(visualMode == kvmNormal);
    actionFileSaveAs->setEnabled(true);
}


void MainWindow::switchToNormalMode()
{
    visualMode = kvmNormal;
    rtViewer->hide();
    updateRouteMenu(false);
    piw->hide();
    daemontb->show();
    layouttb->show();
    layoutedittb->setEnabled(false);
    statusMessage(tr("Layout in normal view mode"));
    propagateViewModeSwitch();
}

void MainWindow::switchToEditLayoutMode()
{
    visualMode = kvmEditLayout;
    daemontb->hide();
    layouttb->hide();
    rtViewer->hide();
    piw->show();
    layoutedittb->setEnabled(true);
    updateRouteMenu(false);
    statusMessage(tr("Entering layout edit mode"));
    propagateViewModeSwitch();
}

void MainWindow::switchToEditRouteMode()
{
    visualMode = kvmEditRoute;
    daemontb->hide();
    layouttb->hide();
    piw->hide();
    layoutedittb->setEnabled(false);
    rtViewer->show();
    updateRouteMenu(true);
    statusMessage(tr("Entering route edit mode"));
    propagateViewModeSwitch();
}

/*
 * called when layout viewmode is changed, visibility of route list
 * window changes and activity state of a selected route changes
 */
void MainWindow::updateRouteMenu(bool rtvIsVisible)
{
    if (rtvIsVisible) {
        updateRouteMenuActivateItems();

        if (visualMode == kvmEditRoute) {
            actionRouteAdd->setEnabled(true);

            bool hasitem = rtViewer->hasCurrentItem();
            actionRouteEdit->setEnabled(hasitem);
            actionRouteCopy->setEnabled(hasitem);
            actionRouteDelete->setEnabled(hasitem);
        }
        else {
            actionRouteAdd->setEnabled(false);
            actionRouteEdit->setEnabled(false);
            actionRouteCopy->setEnabled(false);
            actionRouteDelete->setEnabled(false);
        }
    }
    else {
        actionRouteActivate->setEnabled(false);
        actionRouteWithdraw->setEnabled(false);
        actionRouteRelease->setEnabled(false);
        actionRouteAdd->setEnabled(false);
        actionRouteEdit->setEnabled(false);
        actionRouteCopy->setEnabled(false);
        actionRouteDelete->setEnabled(false);
    }
}


/*update state of route activate menu items*/
void MainWindow::updateRouteMenuActivateItems()
{
    // disable route switching while element pointers are not valid
    if (rtViewer->isVisible() && visualMode != kvmEditLayout) {

        int state = rtViewer->getCurrentItemState();

        switch (state) {
            case -1:
                actionRouteActivate->setEnabled(false);
                actionRouteWithdraw->setEnabled(false);
                actionRouteRelease->setEnabled(false);
                break;
            case 0:
                actionRouteActivate->setEnabled(true);
                actionRouteWithdraw->setEnabled(false);
                actionRouteRelease->setEnabled(false);
                break;
            default:
                actionRouteActivate->setEnabled(false);
                actionRouteWithdraw->setEnabled(true);
                actionRouteRelease->setEnabled(true);
                break;
        }
    }
    else {
        actionRouteActivate->setEnabled(false);
        actionRouteWithdraw->setEnabled(false);
        actionRouteRelease->setEnabled(false);
    }
}


/* update menu items if route list gets empty*/
void MainWindow::updateRouteListMenuItems()
{
    updateRouteMenu(rtViewer->isVisible());
}

/*
 * Open browser window with URL
 */
void MainWindow::runBrowserUrl(const QString& url)
{
    QStringList arguments;
    QString program;

    if (pref.browser.contains(' ')) {
        program = pref.browser.section(' ', 0, 0);
        arguments << pref.browser.section(' ', 1);
    }
    else
        program = pref.browser;

    arguments << url;

    QProcess* proc = new QProcess(this);
    if (proc != NULL)
        proc->start(program, arguments);
}

/*
 * Open handbook in external browser
 */
void MainWindow::slotAboutHelp()
{
    QString langenv, sURL;

    if (QLocale::system().name().startsWith("de"))
        langenv = "de";
    else
        langenv = "en";

    sURL = QString("%1/%2/index.html")
        .arg(HTML_DOC_DIR)
        .arg(langenv);

    runBrowserUrl(sURL);
}

/*
 * Open spdrs60 web resources with external browser
 */
void MainWindow::slotAboutWeb()
{
    runBrowserUrl("http://spdrs60.sourceforge.net/");
}

/*
 * show an original DB clock with minute delay
 */
void MainWindow::slotShowClock()
{
    QProcess* proc = new QProcess(this);
    if (proc != NULL)
        proc->start("centralclock");
}


/*
 * show feedback module window
 */
void MainWindow::slotShowModules()
{
    if (fbViewer != NULL) {
        if (fbViewer->isVisible())
            fbViewer->hide();
        else {
            fbViewer->show();
            fbViewer->raise();
        }
    }
}


/* show a simple keyboard */
void MainWindow::slotViewKeyboard()
{
    if (keybWindow != NULL) {
        if (keybWindow->isVisible())
            keybWindow->hide();
        else {
            keybWindow->show();
            keybWindow->raise();
        }
    }
    else {
        keybWindow = new keyboard(pref.keyboardprot,
                this, "Keyboard");
        connect(keybWindow, SIGNAL(sendSrcpMessage(SrcpMessage*)),
                this, SLOT(sendSrcpMessage(SrcpMessage*)));
        connect(keybWindow, SIGNAL(protocolSelected(int)),
                this, SLOT(saveKeyboardProtocol(int)));

        keybWindow->move(QCursor::pos());
        keybWindow->show();
    }
}

/* show train number input dialog */
void MainWindow::slotViewTrainNumberDialog()
{
    if (trainnumberdialog != NULL) {
        if (trainnumberdialog->isVisible())
            trainnumberdialog->hide();
        else {
            trainnumberdialog->show();
            trainnumberdialog->raise();
        }
    }
    else {
        trainnumberdialog = new TrainNumberDialog(this, "trainnumberdialog");
        connect(trainnumberdialog,
                SIGNAL(trainNumberChanged(unsigned int, unsigned int)),
                router, SLOT(changeTrainNumber(unsigned int, unsigned int)));
        trainnumberdialog->show();
    }
}

/* toggle full screen mode, hide/show also menu, toolbar and statusline */
void MainWindow::slotViewToggleFullscreen()
{
    bool fullscreen = (windowState() & Qt::WindowFullScreen);
    actionViewMenu->setChecked(fullscreen);
    actionViewToolbar->setChecked(fullscreen);
    setWindowState(windowState() ^ Qt::WindowFullScreen);
}

/*
 * save keyboard protocol settings for next time usage
 */
void MainWindow::saveKeyboardProtocol(int protocol)
{
    pref.keyboardprot = protocol;
}


/* show dialog window with user preferences */
void MainWindow::slotEditOptions()
{
    optionsDialog* optDlg = new optionsDialog(this);
    if (optDlg == NULL)
        return;

    optDlg->setPreferences(pref);

    if (optDlg->exec() == QDialog::Accepted) {
        bool al = pref.addresslabeling;
        bool et = pref.datatooltips;

        optDlg->getPreferences(pref);
        writeConfigFile();

        fbViewer->updateBusAndModuleStructure();
        messageHistory->enableTime(pref.showtime);

        if ((al != pref.addresslabeling) || (et != pref.datatooltips))
            emit repaintLayout();
    }
    delete optDlg;
}


/*create new application window*/
void MainWindow::slotFileNewWin()
{
    MainWindow *sw = new MainWindow();
    Q_CHECK_PTR(sw);
    sw->resize(740, 480);
    sw->show();
}


/*create new application window and open file*/
void MainWindow::openFileWindow(const QString& fn)
{
    MainWindow *sw = new MainWindow();
    Q_CHECK_PTR(sw);
    sw->resize(740, 480);
    sw->show();
    sw->openFile(fn);
}


/*
 * create a new locator window and connect its signals directly to gbs
 */
void MainWindow::slotEditFind()
{
    Finder* findWindow = new Finder(this);
    Q_CHECK_PTR(findWindow);
    if (findWindow->exec() == QDialog::Accepted) {
        emit findElement(findWindow->getSearchText(),
                findWindow->getDataType(),
                findWindow->getMatchType());
    }
    delete findWindow;
}


void MainWindow::layoutChangeSize()
{
    newLayoutDialog* nlDlg = new newLayoutDialog(this);
    if (nlDlg == NULL)
        return;

    nlDlg->setWindowTitle(tr("Change layout settings"));
    nlDlg->setColumns(gbs->getColumns());
    nlDlg->setRows(gbs->getRows());
    nlDlg->setLayoutId(gbs->getLayoutId());
    nlDlg->setLayoutName(gbs->getLayoutName());
    nlDlg->setHost(commandPort->getHostname());
    nlDlg->setPort(commandPort->getPortNumber());
    nlDlg->setAutoLogin(cmdAutoLogin);
    nlDlg->setAutoPower(cmdAutoPower);
    nlDlg->setAutoSendAll(cmdAutoSendAll);

    nlDlg->setSwitchboxId(rwccid);
    nlDlg->setSwitchboxName(rwccname);
    nlDlg->setLayoutId(gbs->getLayoutId());
    nlDlg->setLayoutName(gbs->getLayoutName());

    if (nlDlg->exec() == QDialog::Accepted) {
        gbs->setLayoutSize(nlDlg->getColumns(), nlDlg->getRows());
        gbs->setLayoutId(nlDlg->getLayoutId());
        gbs->setLayoutName(nlDlg->getLayoutName());

        commandPort->setServer(nlDlg->getHost(), nlDlg->getPort());
        infoPort->setServer(nlDlg->getHost(), nlDlg->getPort());

        cmdAutoLogin = nlDlg->getAutoLogin();
        cmdAutoPower = nlDlg->getAutoPower();
        cmdAutoSendAll = nlDlg->getAutoSendAll();

        rwccid = nlDlg->getSwitchboxId();
        rwccname = nlDlg->getSwitchboxName();
    }
    delete nlDlg;
}


void MainWindow::layoutSendAll()
{
    qApp->processEvents();
    gbs->slotSendAll();
}


/*check for changed file data*/
bool MainWindow::isModified()
{
    return (gbs->isModified() || router->isModified());
}


/*add a route*/
void MainWindow::slotRouteAdd()
{
    if (rtViewer != NULL)
        rtViewer->slotRouteAdd();
}


/*delete a route*/
void MainWindow::slotRouteDelete()
{
    if (rtViewer != NULL)
        rtViewer->slotRouteDelete();
}

/*TODO: for FDL info
 * cstdlib,
 * #ifdef LINUX
void MainWindow::initUserInfo()
{
    struct passwd* pwentry;
    char* user;

    user = getenv("USER");
    if (user != NULL) {
        pwentry = getpwnam(user);

        if (pwentry != NULL)
            username = pwentry->pw_gecos;
        else
            qWarning("Error reading passwd entry.")
    }
    else
        qWarning("Error reading environment variable USER.")
}
*/

void MainWindow::setupRecentFilesMenu()
{
    fileRecentlyOpenedFiles->clear();

    if (recentFiles.count() > 0) {
        fileRecentlyOpenedFiles->setEnabled(true);
        QStringList::Iterator it = recentFiles.begin();
        for (; it != recentFiles.end(); ++it) {
            fileRecentlyOpenedFiles->addAction(*it);
        }
    }
    else
        fileRecentlyOpenedFiles->setEnabled(false);
}


void MainWindow::recentFileActivated(QAction *action)
{
    if (!action->text().isEmpty()) {
        if (isSave())
            openFile(action->text());
    }
}


void MainWindow::addRecentlyOpenedFile(const QString &fn, QStringList &lst)
{
    QFileInfo fi(fn);
    QString path = fi.absoluteFilePath();
    int index = lst.indexOf(path);

    // already first entry
    if (index == 0)
        return;

    // not first entry but found
    else if (index > 0)
        lst.removeAt(index);

    // not found and new entry necessary
    else  {
        if (lst.count() >= 6)
        lst.pop_back();
    }

    lst.prepend(path);
}

/*Respond to bus power switch action activation*/
void MainWindow::switchBusPower(QAction *ac)
{
    unsigned int bus = ac->data().toUInt();
    bool ison = ac->isChecked();
    if (ison)
        ac->setIcon(QPixmap(layoutstop_xpm));
    else
        ac->setIcon(QPixmap(layoutstart_xpm));

    // menu checkstate update on info arrive
    PowerMessage sm = PowerMessage(SrcpMessage::mtPowerSet, bus, ison);
    sendSrcpMessage(&sm);
}

/*Add new menu item to switch power of available bus*/
void MainWindow::addBusPowerItem(unsigned int bus)
{
    QAction * pa = new QAction(QPixmap(layoutstart_xpm),
            tr("Bus &%1").arg(bus), layoutSwitchBusPower);
    pa->setToolTip(tr("Switch power of bus %1").arg(bus));
    pa->setData(bus);
    pa->setCheckable(true);
    pa->setChecked(false);
    layoutSwitchBusPower->addAction(pa);
    powertb->addAction(pa);

    if (!layoutSwitchBusPower->isEnabled())
        layoutSwitchBusPower->setEnabled(true);
}
/*
 * respond to incomming Generic Messages with "RWCC" actor
 */
void MainWindow::processGenericMessage(unsigned int sendto,
        unsigned int replyto, const CrcfMessage* cm)
{
    if (NULL == cm)
        return;

    if (rwccid != cm->getActorId())
        return;

    QString cms = "";

    switch (cm->getMethod()) {

        /* RWCC <rwccid> GET <attribute> */
        case CrcfMessage::meGet:
            cms = getCrcfInfoMessage(cm->getAttribute());
            if (cms.isEmpty())
                emit statusMessage(tr("Unsupported CRCF attribute '%1' "
                            "detected.").arg(cm->getAttributeStr()));
            else
                sendGmCrcfMessage(sendto, replyto, cms);
            break;

        /* RWCC <rwccid> SET <attribute> */
        case CrcfMessage::meSet:
            // handleCrcfSet();
            switch (cm->getAttribute()) {
                // handleSetVisualMode(mode);
                /* RWCC <rwccid> SET MODE <attribute value> */
                case CrcfMessage::atMode:
                    {
                        elemVisualMode mode = (elemVisualMode)cm->getAttValue();
                        if (mode != visualMode)
                            switch (mode) {
                                case kvmNormal:
                                    switchToNormalMode();
                                    cms = getCrcfInfoMessage(CrcfMessage::atMode);
                                    if (!cms.isEmpty())
                                        sendGmCrcfMessage(sendto, replyto, cms);
                                    else
                                        emit statusMessage(tr("Error assembling "
                                                    "CRCF message for RWCC "
                                                    "MODE '%1'.").arg(mode));
                                    break;
                                case kvmEditLayout:
                                    switchToEditLayoutMode();
                                    cms = getCrcfInfoMessage(CrcfMessage::atMode);
                                    if (!cms.isEmpty())
                                        sendGmCrcfMessage(sendto, replyto, cms);
                                    else
                                        emit statusMessage(tr("Error assembling "
                                                    "CRCF message for RWCC "
                                                    "MODE '%1'.").arg(mode));
                                    break;
                                case kvmEditRoute:
                                    switchToEditRouteMode();
                                    cms = getCrcfInfoMessage(CrcfMessage::atMode);
                                    if (!cms.isEmpty())
                                        sendGmCrcfMessage(sendto, replyto, cms);
                                    else
                                        emit statusMessage(tr("Error assembling "
                                                    "CRCF message for RWCC "
                                                    "MODE '%1'.").arg(mode));
                                    break;
                                default:
                                    emit statusMessage(tr("Unvalid RWCC "
                                                "MODE value '%1' "
                                                "detected.").arg(mode));
                                    break;
                            }
                        break;
                    }
                default:
                    emit statusMessage(tr("Uneditable CRCF RWCC attribute '%1' "
                                "detected.").arg(cm->getAttributeStr()));
                    break;
            }
            break;

        /* RWCC <rwccid> INFO <attribute> */
        case CrcfMessage::meInfo:
            break;

        default:
            statusMessage(tr("Unsupported CRCF method '%1' "
                        "detected.").arg(cm->getMethodStr()));
            break;
    }
}

/*assemble CRCF layout info message string*/
QString MainWindow::getCrcfInfoMessage(CrcfMessage::CrcfAttribute at) const
{
    unsigned int result = 0;

    switch (at) {
        case CrcfMessage::atId:
            result = rwccid;
            break;

        case CrcfMessage::atName:
            return CrcfMessage::message(CrcfMessage::acRwcc, rwccid,
                    CrcfMessage::meInfo, at, rwccname);
            break;

        case CrcfMessage::atMode:
            result = visualMode;
            break;
/*
        case CrcfMessage::atFdl:
            {
            QString fdl = getFdlName();
            return CrcfMessage::message(CrcfMessage::acRwcc, rwccid,
                    CrcfMessage::meInfo, at, fdl);
            break;
            }
*/
        default:
            return "";
            break;
    }

    return CrcfMessage::message(CrcfMessage::acRwcc, rwccid,
            CrcfMessage::meInfo, at, result);
}

/* send a SRCP GM CRCF message to server, method identical to router and
 * gbsarea*/
void MainWindow::sendGmCrcfMessage(unsigned int sendto,
        unsigned int replyto, const QString& cms)
{
    GmMessage sm = GmMessage(SrcpMessage::mtGmSet, sendto, replyto,
            "CRCF", cms);
    sendSrcpMessage(&sm); // emit
}


void MainWindow::runGaInitSequence()
{
    if (!gbs->runSRCP08GAInitSequence()) {
        SRCPCommandState = srcpConnected;

        if (cmdAutoPower) {
            SRCPCommandState = srcp08SetBusPower;
            runSetBusPowerSequence();
        }

        if (cmdAutoSendAll)
            layoutSendAll();
    }
}

/*
 * walk through bus list step by step
 * keep SRCPCommandState while power switching is not finished
 */
void MainWindow::runSetBusPowerSequence()
{
    if (!gbs->setSRCP08BusPower(true)) {
        SRCPCommandState = srcpConnected;
        if (cmdAutoPower && cmdAutoSendAll)
            layoutSendAll();
    }
}
