/***************************************************************************
                           routeelementlvi.cpp
                           version 0.5.5 $Revision: 1.13 $
                           -------------------------------
    copyright            : (C) 2005-2009 by Guido Scholz
    email                : guido.scholz@bayernline.de
    last modified        : $Date: 2009-03-08 08:24:06 $
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
   this code implements a route element displayed as listview item
 ***************************************************************************/

#include "routeelementlvi.h"


RouteElementLVI::RouteElementLVI(QTreeWidget* parent, int index,
    stateElement* se): QTreeWidgetItem(parent)
{
   if (se != NULL) {
       routeElement = *se;
   }
   else {
       routeElement.name = "";
       routeElement.bus = 1;
       routeElement.address = 0;
       routeElement.state = 0;
       routeElement.elemPtr = NULL;
   }

   setText(0, QString::number(index));  
   setText(1, routeElement.name);  
   setText(2, QString::number(routeElement.bus));  
   setText(3, QString::number(routeElement.address));  
   setText(4, QString::number(routeElement.state));

   setTextAlignment(0, Qt::AlignCenter);
   setTextAlignment(2, Qt::AlignRight);
   setTextAlignment(3, Qt::AlignRight);
   setTextAlignment(4, Qt::AlignCenter);
}


void RouteElementLVI::setStateElementData(const stateElement* se)
{
   if (se == NULL)
       return;

   routeElement = *se;

   setText(1, routeElement.name);  
   setText(2, QString::number(routeElement.bus));  
   setText(3, QString::number(routeElement.address));  
   setText(4, QString::number(routeElement.state));  
}


void RouteElementLVI::getStateElementData(stateElement* se) const
{
   if (se == NULL)
       return;

   *se = routeElement;
}

/*
 * sort all columns:
 *
 * No Name      Type
 * -------------------------
 * 0  No        int
 * 1  Name      QString
 * 2  SRCP-Bus  unsigned int
 * 3  Address   unsigned int
 * 4  State     unsigned int
 * -------------------------
 */
bool RouteElementLVI::operator<(const QTreeWidgetItem& other) const
{
    int column = treeWidget()->sortColumn();
    bool returnvalue = false;
    stateElement ce;

    static_cast<const RouteElementLVI&>(other).getStateElementData(&ce);

    switch (column) {
        case 0:
            /* 0 No        int*/
            returnvalue = text(0) < other.text(0);
            break;
        case 1:
            /* 1 Name      QString*/
            returnvalue = (routeElement.name.localeAwareCompare(ce.name) < 0);
            break;
        case 2:
            /* 2 SRCP-Bus  unsigned int*/
            returnvalue = routeElement.bus < ce.bus;
            break;
        case 3:
            /* 3 Address   unsigned int*/
            returnvalue = routeElement.address < ce.address;
            break;
        case 4:
            /* 4 State     unsigned int*/
            returnvalue = routeElement.state < ce.state;
            break;
    }
    return returnvalue;
}

