/***************************************************************************
                           routelistview.cpp
                           version 0.5.3 $Revision: 1.11 $
                           -------------------------------
    copyright            : (C) 2007-2008 by Guido Scholz
    email                : guido.scholz@bayernline.de
    last modified        : $Date: 2008-11-05 08:42:40 $
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *  This program is free software; you can redistribute it and/or modify   *
 *  it under the terms of the GNU General Public License as published by   *
 *  the Free Software Foundation; either version 2 of the License, or      *
 *  (at your option) any later version.                                    *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
   this code implements a listview item displaying route data
 ***************************************************************************/

#include <QHeaderView>

#include "route.h"
#include "routelistview.h"
#include "routelvi.h"


RouteListView::RouteListView(QWidget* parent, const char* name):
    QTreeWidget(parent)
{
    setObjectName(name);
    setUniformRowHeights(true);
    setAllColumnsShowFocus(true);
    setRootIsDecorated(false);

    QTreeWidgetItem* headerItem = new QTreeWidgetItem(
            QStringList() << tr("Id") << tr("Name") << tr("Entry sig.")
            << tr("Entry trk.") << tr("Exit sig.") << tr("Exit trk.")
            << tr("Type") << tr("Train") << tr("State")
            << tr("Length") << tr("Speedlimit"),
            QTreeWidgetItem::UserType + 1);

    headerItem->setTextAlignment(rlviEntryTrack, Qt::AlignRight);
    headerItem->setTextAlignment(rlviExitTrack, Qt::AlignRight);
    headerItem->setTextAlignment(rlviType, Qt::AlignCenter);
    headerItem->setTextAlignment(rlviId, Qt::AlignRight);
    headerItem->setTextAlignment(rlviTrain, Qt::AlignRight);
    headerItem->setTextAlignment(rlviState, Qt::AlignRight);
    headerItem->setTextAlignment(rlviLength, Qt::AlignRight);
    headerItem->setTextAlignment(rlviLimit, Qt::AlignRight);
    setHeaderItem(headerItem);
    header()->resizeSection(rlviFromSig, 70);
    header()->resizeSection(rlviToSig, 70);
    header()->resizeSection(rlviEntryTrack, 70);
    header()->resizeSection(rlviExitTrack, 70);
    header()->resizeSection(rlviType, 50);
    header()->resizeSection(rlviId, 50);
    header()->resizeSection(rlviTrain, 60);
    header()->resizeSection(rlviState, 60);
    header()->resizeSection(rlviLength, 60);
    header()->resizeSection(rlviLimit, 70);
    header()->setStretchLastSection(false);

#if QT_VERSION >= 0x050000
    header()->setSectionResizeMode(QHeaderView::Interactive);
#else
    header()->setResizeMode(QHeaderView::Interactive);
#endif

    // sort list by route name
    sortByColumn(rlviName, Qt::AscendingOrder);
    /*TODO: should be done after list is complete*/
    setSortingEnabled(true);
}


RouteLVI* RouteListView::getRouteLVIByRoute(Route* sr)
{
    if (sr == NULL)
        return NULL;

    RouteLVI* lvi;

    QTreeWidgetItemIterator it(this);
    while (*it) {
        lvi = static_cast<RouteLVI*>(*it);
        if (lvi->getRoute() == sr)
            return lvi;
        ++it;
    }
    return NULL;
}


void RouteListView::keyPressEvent(QKeyEvent* e)
{
    switch (e->key()) {
        case Qt::Key_Insert:
            emit insertPressed();
            e->accept();
            break;
        case Qt::Key_Delete:
            if (currentItem() != NULL)
                emit deletePressed(currentItem());
            e->accept();
            break;
        case Qt::Key_Space:
            if (currentItem() != NULL)
                emit spacePressed(currentItem());
            e->accept();
            break;
        case Qt::Key_Enter:
            if (currentItem() != NULL)
                emit enterPressed(currentItem());
            e->accept();
            break;
        default:
            QTreeWidget::keyPressEvent(e);
    }
}

